@@ -51,8 +51,9 @@
 #include "i915_drm.h"
 #include "intel_chipset.h"
 #include "intel_io.h"
-#include "igt_gt.h"
 #include "igt_debugfs.h"
+#include "igt_gt.h"
+#include "igt_kmod.h"
 #include "version.h"
 #include "config.h"
 #include "intel_reg.h"
@@ -121,6 +122,11 @@ static bool is_virtio_device(int fd)
 	return __is_device(fd, "virt");
 }
 
+static bool is_amd_device(int fd)
+{
+	return __is_device(fd, "amdg");
+}
+
 static bool has_known_intel_chipset(int fd)
 {
 	struct drm_i915_getparam gp;
@@ -167,7 +173,7 @@ void gem_quiescent_gpu(int fd)
 	gem_write(fd, obj.handle, 0, &bbe, sizeof(&bbe));
 
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)&obj;
+	execbuf.buffers_ptr = to_user_pointer(&obj);
 	execbuf.buffer_count = 1;
 
 	for (ring = 0; ring < 1<<6; ring++) {
@@ -183,7 +189,7 @@ void gem_quiescent_gpu(int fd)
 	gem_sync(fd, obj.handle);
 	gem_close(fd, obj.handle);
 
-	igt_drop_caches_set(DROP_RETIRE | DROP_FREED);
+	igt_drop_caches_set(fd, DROP_RETIRE | DROP_FREED);
 }
 
 /**
@@ -228,9 +234,7 @@ int drm_get_card(void)
 
 static int modprobe(const char *driver)
 {
-	char buf[128];
-	snprintf(buf, sizeof(buf), "/sbin/modprobe -s %s", driver);
-	return system(buf);
+	return igt_kmod_load(driver, "");
 }
 
 /**
@@ -272,6 +276,9 @@ int __drm_open_driver(int chipset)
 		    is_virtio_device(fd))
 			return fd;
 
+		if (chipset & DRIVER_AMDGPU && is_amd_device(fd))
+			return fd;
+
 		/* Only VGEM-specific tests should be run on VGEM */
 		if (chipset == DRIVER_ANY && !is_vgem_device(fd))
 			return fd;
