@@ -25,6 +25,7 @@
 #include <inttypes.h>
 #include <sys/stat.h>
 #include <sys/mount.h>
+#include <dirent.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -83,11 +84,22 @@
  * General debugfs helpers
  */
 
-typedef struct {
-	char root[128];
-	char dri_path[128];
-} igt_debugfs_t;
+static bool is_mountpoint(const char *path)
+{
+	char buf[strlen(path) + 4];
+	dev_t dot_dev, dotdot_dev;
+	struct stat st;
 
+	igt_assert_lt(snprintf(buf, sizeof(buf), "%s/.", path), sizeof(buf));
+	igt_assert_eq(stat(buf, &st), 0);
+	dot_dev = st.st_dev;
+
+	igt_assert_lt(snprintf(buf, sizeof(buf), "%s/..", path), sizeof(buf));
+	igt_assert_eq(stat(buf, &st), 0);
+	dotdot_dev = st.st_dev;
+
+	return dot_dev != dotdot_dev;
+}
 
 /**
  * igt_debugfs_mount:
@@ -108,44 +120,78 @@ const char *igt_debugfs_mount(void)
 	if (stat("/sys/kernel/debug/dri", &st) == 0)
 		return "/sys/kernel/debug";
 
-	igt_assert(stat("/sys/kernel/debug", &st) == 0);
-	igt_assert(mount("debug", "/sys/kernel/debug", "debugfs", 0, 0) == 0);
+	igt_assert(is_mountpoint("/sys/kernel/debug") ||
+		   mount("debug", "/sys/kernel/debug", "debugfs", 0, 0) == 0);
+
 	return "/sys/kernel/debug";
 }
 
-static bool __igt_debugfs_init(igt_debugfs_t *debugfs)
+/**
+ * igt_debugfs_dir:
+ * @device: fd of the device
+ *
+ * This opens the debugfs directory corresponding to device for use
+ * with igt_sysfs_get() and related functions.
+ *
+ * Returns:
+ * The directory fd, or -1 on failure.
+ */
+int igt_debugfs_dir(int device)
 {
 	struct stat st;
-	int n;
+	const char *debugfs_root;
+	char path[200];
+	int idx;
 
-	strcpy(debugfs->root, igt_debugfs_mount());
-	for (n = 0; n < 16; n++) {
-		int len = sprintf(debugfs->dri_path, "%s/dri/%d", debugfs->root, n);
-		sprintf(debugfs->dri_path + len, "/i915_error_state");
-		if (stat(debugfs->dri_path, &st) == 0) {
-			debugfs->dri_path[len] = '\0';
-			return true;
+	if (fstat(device, &st)) {
+		igt_debug("Couldn't stat FD for DRM device: %s\n", strerror(errno));
+		return -1;
+	}
+
+	if (!S_ISCHR(st.st_mode)) {
+		igt_debug("FD for DRM device not a char device!\n");
+		return -1;
+	}
+
+	debugfs_root = igt_debugfs_mount();
+
+	idx = minor(st.st_rdev);
+	snprintf(path, sizeof(path), "%s/dri/%d/name", debugfs_root, idx);
+	if (stat(path, &st))
+		return -1;
+
+	if (idx >= 64) {
+		int file, name_len, cmp_len;
+		char name[100], cmp[100];
+
+		file = open(path, O_RDONLY);
+		if (file < 0)
+			return -1;
+
+		name_len = read(file, name, sizeof(name));
+		close(file);
+
+		for (idx = 0; idx < 16; idx++) {
+			snprintf(path, sizeof(path), "%s/dri/%d/name",
+				 debugfs_root, idx);
+			file = open(path, O_RDONLY);
+			if (file < 0)
+				return -1;
+
+			cmp_len = read(file, cmp, sizeof(cmp));
+			close(file);
+
+			if (cmp_len == name_len && !memcmp(cmp, name, name_len))
+				break;
 		}
+
+		if (idx == 16)
+			return -1;
 	}
 
-	debugfs->dri_path[0] = '\0';
-	return false;
-}
-
-static igt_debugfs_t *__igt_debugfs_singleton(void)
-{
-	static igt_debugfs_t singleton;
-	static bool init_done = false;
-
-	if (init_done)
-		return &singleton;
-
-	if (__igt_debugfs_init(&singleton)) {
-		init_done = true;
-		return &singleton;
-	} else {
-		return NULL;
-	}
+	snprintf(path, sizeof(path), "%s/dri/%d", debugfs_root, idx);
+	igt_debug("Opening debugfs directory '%s'\n", path);
+	return open(path, O_RDONLY);
 }
 
 /**
@@ -159,41 +205,19 @@ static igt_debugfs_t *__igt_debugfs_singleton(void)
  * Returns:
  * The Unix file descriptor for the debugfs file or -1 if that didn't work out.
  */
-int igt_debugfs_open(const char *filename, int mode)
+int igt_debugfs_open(int device, const char *filename, int mode)
 {
-	char buf[1024];
-	igt_debugfs_t *debugfs = __igt_debugfs_singleton();
+	int dir, ret;
 
-	if (!debugfs)
-		return -1;
+	dir = igt_debugfs_dir(device);
+	if (dir < 0)
+		return dir;
 
-	sprintf(buf, "%s/%s", debugfs->dri_path, filename);
-	return open(buf, mode);
-}
+	ret = openat(dir, filename, mode);
 
-/**
- * igt_debugfs_fopen:
- * @filename: name of the debugfs node to open
- * @mode: mode string as used by fopen()
- *
- * This opens a debugfs file as a libc FILE. The filename should be
- * relative to the drm device's root, i.e. without "drm/<minor>".
- *
- * Returns:
- * The libc FILE pointer for the debugfs file or NULL if that didn't work out.
- */
-FILE *igt_debugfs_fopen(const char *filename,
-			const char *mode)
-{
-	char buf[1024];
+	close(dir);
 
-	igt_debugfs_t *debugfs = __igt_debugfs_singleton();
-
-	if (!debugfs)
-		return NULL;
-
-	sprintf(buf, "%s/%s", debugfs->dri_path, filename);
-	return fopen(buf, mode);
+	return ret;
 }
 
 /**
@@ -206,21 +230,17 @@ FILE *igt_debugfs_fopen(const char *filename,
  * provided buffer, then closes the file. Users should make sure that the buffer
  * provided is big enough to fit the whole file, plus one byte.
  */
-void __igt_debugfs_read(const char *filename, char *buf, int buf_size)
+void __igt_debugfs_read(int fd, const char *filename, char *buf, int buf_size)
 {
-	FILE *file;
-	size_t n_read;
+	int dir;
+	int len;
 
-	file = igt_debugfs_fopen(filename, "r");
-	igt_assert(file);
-
-	n_read = fread(buf, 1, buf_size - 1, file);
-	igt_assert(n_read > 0);
-	igt_assert(feof(file));
-
-	buf[n_read] = '\0';
-
-	igt_assert(fclose(file) == 0);
+	dir = igt_debugfs_dir(fd);
+	len = igt_sysfs_read(dir, filename, buf, buf_size - 1);
+	if (len < 0)
+		len = 0;
+	buf[len] = '\0';
+	close(dir);
 }
 
 /**
@@ -232,24 +252,27 @@ void __igt_debugfs_read(const char *filename, char *buf, int buf_size)
  *
  * Returns: True if the @substring is found to occur in @filename
  */
-bool igt_debugfs_search(const char *filename, const char *substring)
+bool igt_debugfs_search(int device, const char *filename, const char *substring)
 {
 	FILE *file;
 	size_t n = 0;
 	char *line = NULL;
 	bool matched = false;
+	int fd;
 
-	file = igt_debugfs_fopen(filename, "r");
+	fd = igt_debugfs_open(device, filename, O_RDONLY);
+	file = fdopen(fd, "r");
 	igt_assert(file);
 
 	while (getline(&line, &n, file) >= 0) {
-		matched = (strstr(line, substring) != NULL);
+		matched = strstr(line, substring) != NULL;
 		if (matched)
 			break;
 	}
 
 	free(line);
 	fclose(file);
+	close(fd);
 
 	return matched;
 }
@@ -269,7 +292,7 @@ bool igt_debugfs_search(const char *filename, const char *substring)
  * be random testcase failures when different screen contents end up with the
  * same CRC by chance.
  */
-void igt_assert_crc_equal(igt_crc_t *a, igt_crc_t *b)
+void igt_assert_crc_equal(const igt_crc_t *a, const igt_crc_t *b)
 {
 	int i;
 
@@ -289,27 +312,28 @@ void igt_assert_crc_equal(igt_crc_t *a, igt_crc_t *b)
  */
 char *igt_crc_to_string(igt_crc_t *crc)
 {
-	char buf[128];
+	int i;
+	char buf[128] = { 0 };
 
-	igt_assert_eq(crc->n_words, 5);
-
-	sprintf(buf, "%08x %08x %08x %08x %08x", crc->crc[0],
-		crc->crc[1], crc->crc[2], crc->crc[3], crc->crc[4]);
+	for (i = 0; i < crc->n_words; i++)
+		sprintf(buf + strlen(buf), "%08x ", crc->crc[i]);
 
 	return strdup(buf);
 }
 
+#define MAX_CRC_ENTRIES 10
+#define MAX_LINE_LEN (10 + 11 * MAX_CRC_ENTRIES + 1)
+
 /* (6 fields, 8 chars each, space separated (5) + '\n') */
-#define PIPE_CRC_LINE_LEN       (6 * 8 + 5 + 1)
-/* account for \'0' */
-#define PIPE_CRC_BUFFER_LEN     (PIPE_CRC_LINE_LEN + 1)
+#define LEGACY_LINE_LEN       (6 * 8 + 5 + 1)
 
 struct _igt_pipe_crc {
+	int fd;
+	int dir;
 	int ctl_fd;
 	int crc_fd;
-	int line_len;
-	int buffer_len;
 	int flags;
+	bool is_legacy;
 
 	enum pipe pipe;
 	enum intel_pipe_crc_source source;
@@ -340,13 +364,31 @@ static bool igt_pipe_crc_do_start(igt_pipe_crc_t *pipe_crc)
 	/* Stop first just to make sure we don't have lingering state left. */
 	igt_pipe_crc_stop(pipe_crc);
 
-	sprintf(buf, "pipe %s %s", kmstest_pipe_name(pipe_crc->pipe),
-		pipe_crc_source_name(pipe_crc->source));
-	errno = 0;
-	igt_assert_eq(write(pipe_crc->ctl_fd, buf, strlen(buf)), strlen(buf));
-	if (errno != 0)
-		return false;
+	if (pipe_crc->is_legacy)
+		sprintf(buf, "pipe %s %s", kmstest_pipe_name(pipe_crc->pipe),
+			pipe_crc_source_name(pipe_crc->source));
+	else
+		sprintf(buf, "%s", pipe_crc_source_name(pipe_crc->source));
 
+	igt_assert_eq(write(pipe_crc->ctl_fd, buf, strlen(buf)), strlen(buf));
+
+	if (!pipe_crc->is_legacy) {
+		int err;
+
+		sprintf(buf, "crtc-%d/crc/data", pipe_crc->pipe);
+		err = 0;
+
+		pipe_crc->crc_fd = openat(pipe_crc->dir, buf, pipe_crc->flags);
+		if (pipe_crc->crc_fd < 0)
+			err = -errno;
+
+		if (err == -EINVAL)
+			return false;
+
+		igt_assert_eq(err, 0);
+	}
+
+	errno = 0;
 	return true;
 }
 
@@ -358,22 +400,85 @@ static void igt_pipe_crc_pipe_off(int fd, enum pipe pipe)
 	igt_assert_eq(write(fd, buf, strlen(buf)), strlen(buf));
 }
 
-static void igt_pipe_crc_reset(void)
+static void igt_pipe_crc_reset(int drm_fd)
 {
+	struct dirent *dirent;
+	const char *cmd = "none";
+	bool done = false;
+	DIR *dir;
+	int fdir;
 	int fd;
 
-	fd = igt_debugfs_open("i915_display_crc_ctl", O_WRONLY);
+	fdir = igt_debugfs_dir(drm_fd);
+	if (fdir < 0)
+		return;
 
-	igt_pipe_crc_pipe_off(fd, PIPE_A);
-	igt_pipe_crc_pipe_off(fd, PIPE_B);
-	igt_pipe_crc_pipe_off(fd, PIPE_C);
+	dir = fdopendir(fdir);
+	if (!dir) {
+		close(fdir);
+		return;
+	}
 
-	close(fd);
+	while ((dirent = readdir(dir))) {
+		char buf[128];
+
+		if (strcmp(dirent->d_name, "crtc-") != 0)
+			continue;
+
+		sprintf(buf, "%s/crc/control", dirent->d_name);
+		fd = openat(fdir, buf, O_WRONLY);
+		if (fd < 0)
+			continue;
+
+		igt_assert_eq(write(fd, cmd, strlen(cmd)), strlen(cmd));
+		close(fd);
+
+		done = true;
+	}
+	closedir(dir);
+
+	if (!done) {
+		fd = openat(fdir, "i915_display_crtc_ctl", O_WRONLY);
+		if (fd != -1) {
+			igt_pipe_crc_pipe_off(fd, PIPE_A);
+			igt_pipe_crc_pipe_off(fd, PIPE_B);
+			igt_pipe_crc_pipe_off(fd, PIPE_C);
+
+			close(fd);
+		}
+	}
+
+	close(fdir);
 }
 
 static void pipe_crc_exit_handler(int sig)
 {
-	igt_pipe_crc_reset();
+	struct dirent *dirent;
+	char buf[128];
+	DIR *dir;
+	int fd;
+
+	dir = opendir("/dev/dri");
+	if (!dir)
+		return;
+
+	/*
+	 * Try to reset CRC capture for all DRM devices, this is only needed
+	 * for the legacy CRC ABI and can be completely removed once the
+	 * legacy codepaths are removed.
+	 */
+	while ((dirent = readdir(dir))) {
+		if (strncmp(dirent->d_name, "card", 4) != 0)
+			continue;
+
+		sprintf(buf, "/dev/dri/%s", dirent->d_name);
+		fd = open(buf, O_WRONLY);
+
+		igt_pipe_crc_reset(fd);
+
+		close(fd);
+	}
+	closedir(dir);
 }
 
 /**
@@ -383,43 +488,180 @@ static void pipe_crc_exit_handler(int sig)
  * kernel. Uses igt_skip to automatically skip the test/subtest if this isn't
  * the case.
  */
-void igt_require_pipe_crc(void)
+void igt_require_pipe_crc(int fd)
 {
 	const char *cmd = "pipe A none";
-	FILE *ctl;
-	size_t written;
-	int ret;
+	int ctl, written;
 
-	ctl = igt_debugfs_fopen("i915_display_crc_ctl", "r+");
-	igt_require_f(ctl,
-		      "No display_crc_ctl found, kernel too old\n");
-	written = fwrite(cmd, 1, strlen(cmd), ctl);
-	ret = fflush(ctl);
-	igt_require_f((written == strlen(cmd) && ret == 0) || errno != ENODEV,
-		      "CRCs not supported on this platform\n");
+	ctl = igt_debugfs_open(fd, "crtc-0/crc/control", O_RDONLY);
+	if (ctl < 0) {
+		ctl = igt_debugfs_open(fd, "i915_display_crc_ctl", O_WRONLY);
+		igt_require_f(ctl,
+			      "No display_crc_ctl found, kernel too old\n");
 
-	fclose(ctl);
+		written = write(ctl, cmd, strlen(cmd));
+		igt_require_f(written < 0,
+			      "CRCs not supported on this platform\n");
+	}
+	close(ctl);
+}
+
+static void igt_hpd_storm_exit_handler(int sig)
+{
+	int fd = drm_open_driver_master(DRIVER_INTEL);
+
+	/* Here we assume that only one i915 device will be ever present */
+	igt_hpd_storm_reset(fd);
+
+	close(fd);
+}
+
+/**
+ * igt_hpd_storm_set_threshold:
+ * @threshold: How many hotplugs per second required to trigger an HPD storm,
+ * or 0 to disable storm detection.
+ *
+ * Convienence helper to configure the HPD storm detection threshold for i915
+ * through debugfs. Useful for hotplugging tests where HPD storm detection
+ * might get in the way and slow things down.
+ *
+ * If the system does not support HPD storm detection, this function does
+ * nothing.
+ *
+ * See: https://01.org/linuxgraphics/gfx-docs/drm/gpu/i915.html#hotplug
+ */
+void igt_hpd_storm_set_threshold(int drm_fd, unsigned int threshold)
+{
+	int fd = igt_debugfs_open(drm_fd, "i915_hpd_storm_ctl", O_WRONLY);
+	char buf[16];
+
+	if (fd < 0)
+		return;
+
+	igt_debug("Setting HPD storm threshold to %d\n", threshold);
+	snprintf(buf, sizeof(buf), "%d", threshold);
+	igt_assert_eq(write(fd, buf, strlen(buf)), strlen(buf));
+
+	close(fd);
+	igt_install_exit_handler(igt_hpd_storm_exit_handler);
+}
+
+/**
+ * igt_hpd_storm_reset:
+ *
+ * Convienence helper to reset HPD storm detection to it's default settings.
+ * If hotplug detection was disabled on any ports due to an HPD storm, it will
+ * be immediately re-enabled. Always called on exit if the HPD storm detection
+ * threshold was modified during any tests.
+ *
+ * If the system does not support HPD storm detection, this function does
+ * nothing.
+ *
+ * See: https://01.org/linuxgraphics/gfx-docs/drm/gpu/i915.html#hotplug
+ */
+void igt_hpd_storm_reset(int drm_fd)
+{
+	int fd = igt_debugfs_open(drm_fd, "i915_hpd_storm_ctl", O_WRONLY);
+	const char *buf = "reset";
+
+	if (fd < 0)
+		return;
+
+	igt_debug("Resetting HPD storm threshold\n");
+	igt_assert_eq(write(fd, buf, strlen(buf)), strlen(buf));
+
+	close(fd);
+}
+
+/**
+ * igt_hpd_storm_detected:
+ *
+ * Checks whether or not i915 has detected an HPD interrupt storm on any of the
+ * system's ports.
+ *
+ * This function always returns false on systems that do not support HPD storm
+ * detection.
+ *
+ * See: https://01.org/linuxgraphics/gfx-docs/drm/gpu/i915.html#hotplug
+ *
+ * Returns: Whether or not an HPD storm has been detected.
+ */
+bool igt_hpd_storm_detected(int drm_fd)
+{
+	int fd = igt_debugfs_open(drm_fd, "i915_hpd_storm_ctl", O_RDONLY);
+	char *start_loc;
+	char buf[32] = {0}, detected_str[4];
+	bool ret;
+
+	if (fd < 0)
+		return false;
+
+	igt_assert_lt(0, read(fd, buf, sizeof(buf)));
+	igt_assert(start_loc = strstr(buf, "Detected: "));
+	igt_assert_eq(sscanf(start_loc, "Detected: %s\n", detected_str), 1);
+
+	if (strcmp(detected_str, "yes") == 0)
+		ret = true;
+	else if (strcmp(detected_str, "no") == 0)
+		ret = false;
+	else
+		igt_fail_on_f(true, "Unknown hpd storm detection status '%s'\n",
+			      detected_str);
+
+	close(fd);
+	return ret;
+}
+
+/**
+ * igt_require_hpd_storm_ctl:
+ *
+ * Skips the current test if the system does not have HPD storm detection.
+ *
+ * See: https://01.org/linuxgraphics/gfx-docs/drm/gpu/i915.html#hotplug
+ */
+void igt_require_hpd_storm_ctl(int drm_fd)
+{
+	int fd = igt_debugfs_open(drm_fd, "i915_hpd_storm_ctl", O_RDONLY);
+
+	igt_require_f(fd > 0, "No i915_hpd_storm_ctl found in debugfs\n");
+	close(fd);
 }
 
 static igt_pipe_crc_t *
-pipe_crc_new(enum pipe pipe, enum intel_pipe_crc_source source, int flags)
+pipe_crc_new(int fd, enum pipe pipe, enum intel_pipe_crc_source source, int flags)
 {
 	igt_pipe_crc_t *pipe_crc;
 	char buf[128];
+	int debugfs;
+
+	debugfs = igt_debugfs_dir(fd);
+	igt_assert(debugfs != -1);
 
 	igt_install_exit_handler(pipe_crc_exit_handler);
 
 	pipe_crc = calloc(1, sizeof(struct _igt_pipe_crc));
 
-	pipe_crc->ctl_fd = igt_debugfs_open("i915_display_crc_ctl", O_WRONLY);
-	igt_assert(pipe_crc->ctl_fd != -1);
+	sprintf(buf, "crtc-%d/crc/control", pipe);
+	pipe_crc->ctl_fd = openat(debugfs, buf, O_WRONLY);
+	if (pipe_crc->ctl_fd == -1) {
+		pipe_crc->ctl_fd = openat(debugfs,
+					  "i915_display_crc_ctl", O_WRONLY);
+		igt_assert(pipe_crc->ctl_fd != -1);
+		pipe_crc->is_legacy = true;
+	}
 
-	sprintf(buf, "i915_pipe_%s_crc", kmstest_pipe_name(pipe));
-	pipe_crc->crc_fd = igt_debugfs_open(buf, flags);
-	igt_assert(pipe_crc->crc_fd != -1);
+	if (pipe_crc->is_legacy) {
+		sprintf(buf, "i915_pipe_%s_crc", kmstest_pipe_name(pipe));
+		pipe_crc->crc_fd = openat(debugfs, buf, flags);
+		igt_assert(pipe_crc->crc_fd != -1);
+		igt_debug("Using legacy frame CRC ABI\n");
+	} else {
+		pipe_crc->crc_fd = -1;
+		igt_debug("Using generic frame CRC ABI\n");
+	}
 
-	pipe_crc->line_len = PIPE_CRC_LINE_LEN;
-	pipe_crc->buffer_len = PIPE_CRC_BUFFER_LEN;
+	pipe_crc->fd = fd;
+	pipe_crc->dir = debugfs;
 	pipe_crc->pipe = pipe;
 	pipe_crc->source = source;
 	pipe_crc->flags = flags;
@@ -440,9 +682,9 @@ pipe_crc_new(enum pipe pipe, enum intel_pipe_crc_source source, int flags)
  * least INTEL_PIPE_CRC_SOURCE_AUTO everywhere.
  */
 igt_pipe_crc_t *
-igt_pipe_crc_new(enum pipe pipe, enum intel_pipe_crc_source source)
+igt_pipe_crc_new(int fd, enum pipe pipe, enum intel_pipe_crc_source source)
 {
-	return pipe_crc_new(pipe, source, O_RDONLY);
+	return pipe_crc_new(fd, pipe, source, O_RDONLY);
 }
 
 /**
@@ -458,9 +700,9 @@ igt_pipe_crc_new(enum pipe pipe, enum intel_pipe_crc_source source)
  * least INTEL_PIPE_CRC_SOURCE_AUTO everywhere.
  */
 igt_pipe_crc_t *
-igt_pipe_crc_new_nonblock(enum pipe pipe, enum intel_pipe_crc_source source)
+igt_pipe_crc_new_nonblock(int fd, enum pipe pipe, enum intel_pipe_crc_source source)
 {
-	return pipe_crc_new(pipe, source, O_RDONLY | O_NONBLOCK);
+	return pipe_crc_new(fd, pipe, source, O_RDONLY | O_NONBLOCK);
 }
 
 /**
@@ -476,37 +718,65 @@ void igt_pipe_crc_free(igt_pipe_crc_t *pipe_crc)
 
 	close(pipe_crc->ctl_fd);
 	close(pipe_crc->crc_fd);
+	close(pipe_crc->dir);
 	free(pipe_crc);
 }
 
-static bool pipe_crc_init_from_string(igt_crc_t *crc, const char *line)
+static bool pipe_crc_init_from_string(igt_pipe_crc_t *pipe_crc, igt_crc_t *crc,
+				      const char *line)
 {
-	int n;
+	int n, i;
+	const char *buf;
 
-	crc->n_words = 5;
-	n = sscanf(line, "%8u %8x %8x %8x %8x %8x", &crc->frame, &crc->crc[0],
-		   &crc->crc[1], &crc->crc[2], &crc->crc[3], &crc->crc[4]);
-	return n == 6;
+	if (pipe_crc->is_legacy) {
+		crc->has_valid_frame = true;
+		crc->n_words = 5;
+		n = sscanf(line, "%8u %8x %8x %8x %8x %8x", &crc->frame,
+			   &crc->crc[0], &crc->crc[1], &crc->crc[2],
+			   &crc->crc[3], &crc->crc[4]);
+		return n == 6;
+	}
+
+	if (strncmp(line, "XXXXXXXXXX", 10) == 0)
+		crc->has_valid_frame = false;
+	else {
+		crc->has_valid_frame = true;
+		crc->frame = strtoul(line, NULL, 16);
+	}
+
+	buf = line + 10;
+	for (i = 0; *buf != '\n'; i++, buf += 11)
+		crc->crc[i] = strtoul(buf, NULL, 16);
+
+	crc->n_words = i;
+
+	return true;
 }
 
 static int read_crc(igt_pipe_crc_t *pipe_crc, igt_crc_t *out)
 {
 	ssize_t bytes_read;
-	char buf[pipe_crc->buffer_len];
+	char buf[MAX_LINE_LEN + 1];
+	size_t read_len;
+
+	if (pipe_crc->is_legacy)
+		read_len = LEGACY_LINE_LEN;
+	else
+		read_len = MAX_LINE_LEN;
 
 	igt_set_timeout(5, "CRC reading");
-	bytes_read = read(pipe_crc->crc_fd, &buf, pipe_crc->line_len);
+	bytes_read = read(pipe_crc->crc_fd, &buf, read_len);
 	igt_reset_timeout();
 
-	if (bytes_read < 0 && errno == EAGAIN) {
+	if (bytes_read < 0 && errno == EAGAIN)
 		igt_assert(pipe_crc->flags & O_NONBLOCK);
+
+	if (bytes_read < 0)
 		bytes_read = 0;
-	} else {
-		igt_assert_eq(bytes_read, pipe_crc->line_len);
-	}
+
 	buf[bytes_read] = '\0';
 
-	if (bytes_read && !pipe_crc_init_from_string(out, buf))
+	if (bytes_read && !pipe_crc_init_from_string(pipe_crc, out, buf))
 		return -EINVAL;
 
 	return bytes_read;
@@ -530,15 +800,18 @@ void igt_pipe_crc_start(igt_pipe_crc_t *pipe_crc)
 
 	igt_assert(igt_pipe_crc_do_start(pipe_crc));
 
-	/*
-	 * For some no yet identified reason, the first CRC is bonkers. So
-	 * let's just wait for the next vblank and read out the buggy result.
-	 *
-	 * On CHV sometimes the second CRC is bonkers as well, so don't trust
-	 * that one either.
-	 */
-	read_one_crc(pipe_crc, &crc);
-	read_one_crc(pipe_crc, &crc);
+	if (pipe_crc->is_legacy) {
+		/*
+		 * For some no yet identified reason, the first CRC is
+		 * bonkers. So let's just wait for the next vblank and read
+		 * out the buggy result.
+		 *
+		 * On CHV sometimes the second CRC is bonkers as well, so
+		 * don't trust that one either.
+		 */
+		read_one_crc(pipe_crc, &crc);
+		read_one_crc(pipe_crc, &crc);
+	}
 }
 
 /**
@@ -551,8 +824,14 @@ void igt_pipe_crc_stop(igt_pipe_crc_t *pipe_crc)
 {
 	char buf[32];
 
-	sprintf(buf, "pipe %s none", kmstest_pipe_name(pipe_crc->pipe));
-	igt_assert_eq(write(pipe_crc->ctl_fd, buf, strlen(buf)), strlen(buf));
+	if (pipe_crc->is_legacy) {
+		sprintf(buf, "pipe %s none", kmstest_pipe_name(pipe_crc->pipe));
+		igt_assert_eq(write(pipe_crc->ctl_fd, buf, strlen(buf)),
+			      strlen(buf));
+	} else {
+		close(pipe_crc->crc_fd);
+		pipe_crc->crc_fd = -1;
+	}
 }
 
 /**
@@ -652,6 +931,26 @@ void igt_pipe_crc_collect_crc(igt_pipe_crc_t *pipe_crc, igt_crc_t *out_crc)
  * Drop caches
  */
 
+/**
+ * igt_drop_caches_has:
+ * @val: bitmask for DROP_* values
+ *
+ * This queries the debugfs to see if it supports the full set of desired
+ * operations.
+ */
+bool igt_drop_caches_has(int drm_fd, uint64_t val)
+{
+	uint64_t mask;
+	int dir;
+
+	mask = 0;
+	dir = igt_debugfs_dir(drm_fd);
+	igt_sysfs_scanf(dir, "i915_gem_drop_caches", "0x%" PRIx64, &mask);
+	close(dir);
+
+	return (val & mask) == val;
+}
+
 /**
  * igt_drop_caches_set:
  * @val: bitmask for DROP_* values
@@ -659,7 +958,7 @@ void igt_pipe_crc_collect_crc(igt_pipe_crc_t *pipe_crc, igt_crc_t *out_crc)
  * This calls the debugfs interface the drm/i915 GEM driver exposes to drop or
  * evict certain classes of gem buffer objects.
  */
-void igt_drop_caches_set(uint64_t val)
+void igt_drop_caches_set(int drm_fd, uint64_t val)
 {
 	int fd;
 	char data[19];
@@ -667,7 +966,7 @@ void igt_drop_caches_set(uint64_t val)
 
 	sprintf(data, "0x%" PRIx64, val);
 
-	fd = igt_debugfs_open("i915_gem_drop_caches", O_WRONLY);
+	fd = igt_debugfs_open(drm_fd, "i915_gem_drop_caches", O_WRONLY);
 
 	igt_assert(fd >= 0);
 	do {
@@ -733,17 +1032,17 @@ void igt_enable_prefault(void)
 	igt_prefault_control(true);
 }
 
-static int get_object_count(void)
+static int get_object_count(int fd)
 {
-	FILE *file;
-	int ret, scanned;
+	int dir, ret, scanned;
 
-	igt_drop_caches_set(DROP_RETIRE | DROP_ACTIVE | DROP_FREED);
+	igt_drop_caches_set(fd, DROP_RETIRE | DROP_ACTIVE | DROP_FREED);
 
-	file = igt_debugfs_fopen("i915_gem_objects", "r");
-
-	scanned = fscanf(file, "%i objects", &ret);
+	dir = igt_debugfs_dir(fd);
+	scanned = igt_sysfs_scanf(dir, "i915_gem_objects",
+				  "%i objects", &ret);
 	igt_assert_eq(scanned, 1);
+	close(dir);
 
 	return ret;
 }
@@ -760,7 +1059,7 @@ int igt_get_stable_obj_count(int driver)
 {
 	int obj_count;
 	gem_quiescent_gpu(driver);
-	obj_count = get_object_count();
+	obj_count = get_object_count(driver);
 	/* The test relies on the system being in the same state before and
 	 * after the test so any difference in the object count is a result of
 	 * leaks during the test. gem_quiescent_gpu() mostly achieves this but
@@ -774,7 +1073,7 @@ int igt_get_stable_obj_count(int driver)
 		while (loop_count < 4) {
 			usleep(200000);
 			gem_quiescent_gpu(driver);
-			obj_count = get_object_count();
+			obj_count = get_object_count(driver);
 			if (obj_count == prev_obj_count) {
 				loop_count++;
 			} else {
@@ -790,33 +1089,6 @@ int igt_get_stable_obj_count(int driver)
 	return obj_count;
 }
 
-
-/* Non-i915 specific debugfs API */
-
-/**
- * igt_debugfs_dir:
- * @device: fd of the device (or -1 to default to Intel)
- *
- * This opens the debugfs directory corresponding to device for use
- * with igt_sysfs_get() and related functions.
- *
- * Returns:
- * The directory fd, or -1 on failure.
- */
-int igt_debugfs_dir(int device)
-{
-	struct stat st;
-	char path[256];
-
-	if (fstat(device, &st) || !S_ISCHR(st.st_mode))
-		return -1;
-
-	sprintf(path, "%s/dri/%d",
-		igt_debugfs_mount(), (int)(st.st_rdev & 0xff));
-	igt_debug("Opening debugfs dir %s\n", path);
-	return open(path, O_RDONLY);
-}
-
 void igt_debugfs_dump(int device, const char *filename)
 {
 	char *contents;
