@@ -22,6 +22,7 @@
  */
 
 #include <string.h>
+#include <strings.h>
 #include <signal.h>
 #include <errno.h>
 #include <sys/types.h>
@@ -39,8 +40,6 @@
 #include "intel_reg.h"
 #include "intel_chipset.h"
 
-#define LOCAL_CONTEXT_PARAM_NO_ERROR_CAPTURE 0x4
-
 /**
  * SECTION:igt_gt
  * @short_description: GT support library
@@ -145,6 +144,8 @@ static void context_set_ban(int fd, unsigned ctx, unsigned ban)
 {
 	struct local_i915_gem_context_param param;
 
+	memset(&param, 0, sizeof(param));
+	param.context = ctx;
 	param.value = ban;
 	param.size = 0;
 	param.param = LOCAL_CONTEXT_PARAM_BANNABLE;
@@ -261,7 +262,7 @@ igt_hang_t igt_hang_ctx(int fd,
 
 	exec.handle = gem_create(fd, 4096);
 	exec.relocation_count = 1;
-	exec.relocs_ptr = (uintptr_t)&reloc;
+	exec.relocs_ptr = to_user_pointer(&reloc);
 
 	memset(b, 0xc5, sizeof(b));
 
@@ -292,7 +293,7 @@ igt_hang_t igt_hang_ctx(int fd,
 	reloc.target_handle = exec.handle;
 	reloc.read_domains = I915_GEM_DOMAIN_COMMAND;
 
-	execbuf.buffers_ptr = (uintptr_t)&exec;
+	execbuf.buffers_ptr = to_user_pointer(&exec);
 	execbuf.buffer_count = 1;
 	execbuf.flags = ring;
 	i915_execbuffer2_set_context_id(execbuf, ctx);
@@ -360,19 +361,20 @@ void igt_post_hang_ring(int fd, igt_hang_t arg)
  * stuck, either because the test manually disabled gpu resets or because the
  * test hit an hangcheck bug
  */
-void igt_force_gpu_reset(void)
+void igt_force_gpu_reset(int drm_fd)
 {
-	int fd, ret;
+	int dir, wedged;
 
 	igt_debug("Triggering GPU reset\n");
 
-	fd = igt_debugfs_open("i915_wedged", O_RDWR);
-	igt_require(fd >= 0);
+	dir = igt_debugfs_dir(drm_fd);
 
-	ret = write(fd, "-1\n", 3);
-	close(fd);
+	igt_sysfs_set(dir, "i915_wedged", "-1");
+	igt_sysfs_scanf(dir, "i915_wedged", "%d", &wedged);
 
-	igt_assert_eq(ret, 3);
+	close(dir);
+
+	igt_assert(!wedged);
 }
 
 /* GPU abusers */
@@ -441,11 +443,11 @@ void igt_stop_hang_helper(void)
  * Returns:
  * The file descriptor of the forcewake handle or -1 if that didn't work out.
  */
-int igt_open_forcewake_handle(void)
+int igt_open_forcewake_handle(int fd)
 {
 	if (getenv("IGT_NO_FORCEWAKE"))
 		return -1;
-	return igt_debugfs_open("i915_forcewake_user", O_WRONLY);
+	return igt_debugfs_open(fd, "i915_forcewake_user", O_WRONLY);
 }
 
 #if defined(__x86_64__) || defined(__i386__)
@@ -528,34 +530,49 @@ void igt_clflush_range(void *addr, int size)
  */
 unsigned intel_detect_and_clear_missed_interrupts(int fd)
 {
-	unsigned missed = 0;
-	FILE *file;
+	unsigned missed;
+	int dir;
 
 	gem_quiescent_gpu(fd);
 
-	file = igt_debugfs_fopen("i915_ring_missed_irq", "r");
-	if (file) {
-		igt_assert(fscanf(file, "%x", &missed) == 1);
-		fclose(file);
-	}
-	if (missed) {
-		file = igt_debugfs_fopen("i915_ring_missed_irq", "w");
-		if (file) {
-			fwrite("0\n", 1, 2, file);
-			fclose(file);
-		}
-	}
+	dir = igt_debugfs_dir(fd);
 
+	missed = 0;
+	igt_assert(igt_sysfs_scanf(dir, "i915_ring_missed_irq", "%x", &missed) == 1);
+	if (missed)
+		igt_sysfs_set(dir, "i915_ring_missed_irq", "0");
+
+	close(dir);
+
+	errno = 0;
 	return missed;
 }
 
 const struct intel_execution_engine intel_execution_engines[] = {
 	{ "default", NULL, 0, 0 },
-	{ "render", "render ring", I915_EXEC_RENDER, 0 },
-	{ "bsd", "bsd ring", I915_EXEC_BSD, 0 },
-	{ "bsd1", "bsd ring", I915_EXEC_BSD, 1<<13 /*I915_EXEC_BSD_RING1*/ },
-	{ "bsd2", "bsd2 ring", I915_EXEC_BSD, 2<<13 /*I915_EXEC_BSD_RING2*/ },
-	{ "blt", "blitter ring", I915_EXEC_BLT, 0 },
-	{ "vebox", "video enhancement ring", I915_EXEC_VEBOX, 0 },
+	{ "render", "rcs0", I915_EXEC_RENDER, 0 },
+	{ "bsd", "vcs0", I915_EXEC_BSD, 0 },
+	{ "bsd1", "vcs0", I915_EXEC_BSD, 1<<13 /*I915_EXEC_BSD_RING1*/ },
+	{ "bsd2", "vcs1", I915_EXEC_BSD, 2<<13 /*I915_EXEC_BSD_RING2*/ },
+	{ "blt", "bcs0", I915_EXEC_BLT, 0 },
+	{ "vebox", "vecs0", I915_EXEC_VEBOX, 0 },
 	{ NULL, 0, 0 }
 };
+
+bool gem_can_store_dword(int fd, unsigned int engine)
+{
+	uint16_t devid = intel_get_drm_devid(fd);
+	const struct intel_device_info *info = intel_get_device_info(devid);
+	const int gen = ffs(info->gen);
+
+	if (gen <= 2) /* requires physical addresses */
+		return false;
+
+	if (gen == 6 && (engine & ~(3<<13)) == I915_EXEC_BSD)
+		return false; /* kills the machine! */
+
+	if (info->is_broadwater)
+		return false; /* Not sure yet... */
+
+	return true;
+}
