@@ -83,7 +83,7 @@
 static char *forced_connectors[MAX_CONNECTORS + 1];
 static int forced_connectors_device[MAX_CONNECTORS + 1];
 
-static void update_edid_csum(unsigned char *edid)
+static void update_edid_csum(unsigned char *edid, int cea_pos)
 {
 	int i, sum = 0;
 	struct tm *tm;
@@ -96,9 +96,9 @@ static void update_edid_csum(unsigned char *edid)
 
 	/* calculate checksum */
 	for (i = 0; i < 127; i++) {
-		sum = sum + edid[i];
+		sum = sum + edid[cea_pos + i];
 	}
-	edid[127] = 256 - sum;
+	edid[cea_pos + 127] = 256 - sum;
 }
 
 #define VFREQ 60
@@ -136,7 +136,7 @@ static void update_edid_csum(unsigned char *edid)
  */
 const unsigned char* igt_kms_get_base_edid(void)
 {
-	update_edid_csum(base_edid);
+	update_edid_csum(base_edid, 0);
 
 	return base_edid;
 }
@@ -306,7 +306,7 @@ igt_atomic_fill_pipe_props(igt_display_t *display, igt_pipe_t *pipe,
  */
 const unsigned char* igt_kms_get_alt_edid(void)
 {
-	update_edid_csum(alt_edid);
+	update_edid_csum(alt_edid, 0);
 
 	return alt_edid;
 }
@@ -747,7 +747,7 @@ void kmstest_force_edid(int drm_fd, drmModeConnector *connector,
 
 	igt_assert_neq(asprintf(&path, "%s-%d/edid_override", kmstest_connector_type_str(connector->connector_type), connector->connector_type_id),
 		       -1);
-	debugfs_fd = igt_debugfs_open(path, O_WRONLY | O_TRUNC);
+	debugfs_fd = igt_debugfs_open(drm_fd, path, O_WRONLY | O_TRUNC);
 	free(path);
 
 	igt_assert(debugfs_fd != -1);
@@ -1116,6 +1116,59 @@ kmstest_get_property(int drm_fd, uint32_t object_id, uint32_t object_type,
 	return found;
 }
 
+struct edid_block {
+    int pos;
+    unsigned char *data;
+};
+
+#define DTD_SUPPORTS_AUDIO 1<<6
+
+static struct edid_block
+init_cea_block(const unsigned char *edid, size_t length,
+	       unsigned char *new_edid_ptr[], size_t *new_length,
+	       char extra_extensions_length,
+	       uint32_t dtd_support)
+{
+	struct edid_block new_edid;
+	int n_extensions;
+	int pos;
+	static const char cea_header_len = 4, video_block_len = 6;
+
+	igt_assert(new_edid_ptr != NULL && new_length != NULL);
+
+	*new_length = length + 128;
+
+	new_edid.data = calloc(*new_length, sizeof(char));
+	igt_assert_f(new_edid.data, "Failed to allocate %zu bytes for edid\n", sizeof(new_length));
+	memcpy(new_edid.data, edid, length);
+	*new_edid_ptr = new_edid.data;
+
+	n_extensions = new_edid.data[126];
+	n_extensions++;
+	new_edid.data[126] = n_extensions;
+
+	update_edid_csum(new_edid.data, 0);
+
+	/* add a cea-861 extension block */
+	pos = length;
+	new_edid.data[pos++] = 0x2;
+	new_edid.data[pos++] = 0x3;
+	new_edid.data[pos++] = cea_header_len + video_block_len +
+		extra_extensions_length;
+	new_edid.data[pos++] = dtd_support;
+
+	/* video block (id | length) */
+	new_edid.data[pos++] = 2 << 5 | (video_block_len - 1);
+	new_edid.data[pos++] = 32 | 0x80; /* 1080p @ 24Hz | (native)*/
+	new_edid.data[pos++] = 5;         /* 1080i @ 60Hz */
+	new_edid.data[pos++] = 20;        /* 1080i @ 50Hz */
+	new_edid.data[pos++] = 4;         /* 720p @ 60Hz*/
+	new_edid.data[pos++] = 19;        /* 720p @ 50Hz*/
+	new_edid.pos = pos;
+
+	return new_edid;
+}
+
 /**
  * kmstest_edid_add_3d:
  * @edid: an existing valid edid block
@@ -1129,72 +1182,135 @@ kmstest_get_property(int drm_fd, uint32_t object_id, uint32_t object_type,
 void kmstest_edid_add_3d(const unsigned char *edid, size_t length,
 			 unsigned char *new_edid_ptr[], size_t *new_length)
 {
-	unsigned char *new_edid;
-	int n_extensions;
-	char sum = 0;
-	int pos;
-	int i;
-	char cea_header_len = 4, video_block_len = 6, vsdb_block_len = 11;
-
-	igt_assert(new_edid_ptr != NULL && new_length != NULL);
-
-	*new_length = length + 128;
-
-	new_edid = calloc(*new_length, sizeof(char));
-	igt_assert_f(new_edid, "Failed to allocate %zu bytes for edid\n", sizeof(new_length));
-	memcpy(new_edid, edid, length);
-	*new_edid_ptr = new_edid;
-
-	n_extensions = new_edid[126];
-	n_extensions++;
-	new_edid[126] = n_extensions;
-
-	/* recompute checksum */
-	for (i = 0; i < 127; i++) {
-		sum = sum + new_edid[i];
-	}
-	new_edid[127] = 256 - sum;
-
-	/* add a cea-861 extension block */
-	pos = length;
-	new_edid[pos++] = 0x2;
-	new_edid[pos++] = 0x3;
-	new_edid[pos++] = cea_header_len + video_block_len + vsdb_block_len;
-	new_edid[pos++] = 0x0;
-
-	/* video block (id | length) */
-	new_edid[pos++] = 2 << 5 | (video_block_len - 1);
-	new_edid[pos++] = 32 | 0x80; /* 1080p @ 24Hz | (native)*/
-	new_edid[pos++] = 5;         /* 1080i @ 60Hz */
-	new_edid[pos++] = 20;        /* 1080i @ 50Hz */
-	new_edid[pos++] = 4;         /* 720p @ 60Hz*/
-	new_edid[pos++] = 19;        /* 720p @ 50Hz*/
+	char vsdb_block_len = 11;
+	struct edid_block new_edid = init_cea_block(edid, length, new_edid_ptr,
+						    new_length, vsdb_block_len,
+						    0);
+	int pos = new_edid.pos;
 
 	/* vsdb block ( id | length ) */
-	new_edid[pos++] = 3 << 5 | (vsdb_block_len - 1);
+	new_edid.data[pos++] = 3 << 5 | (vsdb_block_len - 1);
 	/* registration id */
-	new_edid[pos++] = 0x3;
-	new_edid[pos++] = 0xc;
-	new_edid[pos++] = 0x0;
+	new_edid.data[pos++] = 0x3;
+	new_edid.data[pos++] = 0xc;
+	new_edid.data[pos++] = 0x0;
 	/* source physical address */
-	new_edid[pos++] = 0x10;
-	new_edid[pos++] = 0x00;
+	new_edid.data[pos++] = 0x10;
+	new_edid.data[pos++] = 0x00;
 	/* Supports_AI ... etc */
-	new_edid[pos++] = 0x00;
+	new_edid.data[pos++] = 0x00;
 	/* Max TMDS Clock */
-	new_edid[pos++] = 0x00;
+	new_edid.data[pos++] = 0x00;
 	/* Latency present, HDMI Video Present */
-	new_edid[pos++] = 0x20;
+	new_edid.data[pos++] = 0x20;
 	/* HDMI Video */
-	new_edid[pos++] = 0x80;
-	new_edid[pos++] = 0x00;
+	new_edid.data[pos++] = 0x80;
+	new_edid.data[pos++] = 0x00;
 
-	/* checksum */
-	sum = 0;
-	for (i = 0; i < 127; i++) {
-		sum = sum + new_edid[length + i];
-	}
-	new_edid[length + 127] = 256 - sum;
+	update_edid_csum(new_edid.data, length);
+}
+
+/**
+ * kmstest_edid_add_4k:
+ * @edid: an existing valid edid block
+ * @length: length of @edid
+ * @new_edid_ptr: pointer to where the new edid will be placed
+ * @new_length: pointer to the size of the new edid
+ *
+ * Makes a copy of an existing edid block and adds an extension indicating
+ * a HDMI 4K mode in vsdb.
+ */
+void kmstest_edid_add_4k(const unsigned char *edid, size_t length,
+			 unsigned char *new_edid_ptr[], size_t *new_length)
+{
+	char vsdb_block_len = 12;
+	struct edid_block new_edid = init_cea_block(edid, length, new_edid_ptr,
+						    new_length, vsdb_block_len,
+						    0);
+	int pos = new_edid.pos;
+
+	/* vsdb block ( id | length ) */
+	new_edid.data[pos++] = 3 << 5 | (vsdb_block_len - 1);
+	/* registration id */
+	new_edid.data[pos++] = 0x3;
+	new_edid.data[pos++] = 0xc;
+	new_edid.data[pos++] = 0x0;
+	/* source physical address */
+	new_edid.data[pos++] = 0x10;
+	new_edid.data[pos++] = 0x00;
+	/* Supports_AI ... etc */
+	new_edid.data[pos++] = 0x00;
+	/* Max TMDS Clock */
+	new_edid.data[pos++] = 0x00;
+	/* Latency present, HDMI Video Present */
+	new_edid.data[pos++] = 0x20;
+	/* HDMI Video */
+	new_edid.data[pos++] = 0x00; /* 3D present */
+
+	/* HDMI MODE LEN -- how many entries */
+	new_edid.data[pos++] = 0x20;
+	/* 2160p, specified as short descriptor */
+	new_edid.data[pos++] = 0x01;
+
+	update_edid_csum(new_edid.data, length);
+}
+
+/**
+ * kmstest_edid_add_audio:
+ * @edid: an existing valid edid block
+ * @length: length of @edid
+ * @new_edid_ptr: pointer to where the new edid will be placed
+ * @new_length: pointer to the size of the new edid
+ *
+ * Makes a copy of an existing edid block and adds an extension indicating
+ * basic audio support and speaker data block.
+ *
+ */
+void kmstest_edid_add_audio(const unsigned char *edid, size_t length,
+			    unsigned char *new_edid_ptr[], size_t *new_length)
+{
+	char vsdb_block_len = 10, audio_block_len = 4, spkr_block_len = 4;
+	struct edid_block new_edid = init_cea_block(edid, length, new_edid_ptr,
+						    new_length,
+						    vsdb_block_len +
+						    audio_block_len +
+						    spkr_block_len,
+						    DTD_SUPPORTS_AUDIO);
+	int pos = new_edid.pos;
+
+	/* audio block, short audio block descriptors  */
+	new_edid.data[pos++] = (1 << 5) | (audio_block_len - 1);
+	new_edid.data[pos++] = 0x09; /* Audio Format, PCM */
+	new_edid.data[pos++] = 0x07; /* Frequency, 32, 44.1, 48kHz  */
+	new_edid.data[pos++] = 0x07; /* Bit Rate 16, 20, 24 bit */
+
+
+	/* vsdb block ( id | length ) -- need vsdb as well
+	 * otherwise the kernel will fallback to lower clock modes */
+	new_edid.data[pos++] = 3 << 5 | (vsdb_block_len - 1);
+	/* registration id */
+	new_edid.data[pos++] = 0x3;
+	new_edid.data[pos++] = 0xc;
+	new_edid.data[pos++] = 0x0;
+	/* source physical address */
+	new_edid.data[pos++] = 0x10;
+	new_edid.data[pos++] = 0x00;
+	/* Supports_AI ... etc */
+	new_edid.data[pos++] = 0x00;
+	/* Max TMDS Clock */
+	new_edid.data[pos++] = 0x00;
+	/* Latency present, HDMI Video Present */
+	new_edid.data[pos++] = 0x20;
+	/* HDMI Video */
+	new_edid.data[pos++] = 0x00; /* 3D present */
+
+	/* speaker data block */
+	new_edid.data[pos++] = (4 << 5) | (spkr_block_len - 1);
+	new_edid.data[pos++] = (1 << 5);
+	new_edid.data[pos++] = 0x00;
+	new_edid.data[pos++] = 0x00;
+
+	update_edid_csum(new_edid.data, length);
 }
 
 /**
@@ -1324,34 +1440,34 @@ static void parse_crtc(char *info, struct kmstest_crtc *crtc)
 	igt_assert_eq(ret, 2);
 }
 
-void kmstest_get_crtc(enum pipe pipe, struct kmstest_crtc *crtc)
+void kmstest_get_crtc(int device, enum pipe pipe, struct kmstest_crtc *crtc)
 {
 	char tmp[256];
-	FILE *fid;
-	const char *mode = "r";
+	FILE *file;
 	int ncrtc;
 	int line;
 	long int n;
+	int fd;
 
-	fid = igt_debugfs_fopen("i915_display_info", mode);
-
-	igt_skip_on(fid == NULL);
+	fd = igt_debugfs_open(device, "i915_display_info", O_RDONLY);
+	file = fdopen(fd, "r");
+	igt_skip_on(file == NULL);
 
 	ncrtc = 0;
 	line = 0;
-	while (fgets(tmp, 256, fid) != NULL) {
+	while (fgets(tmp, 256, file) != NULL) {
 		if ((strstr(tmp, "CRTC") != NULL) && (line > 0)) {
 			if (strstr(tmp, "active=yes") != NULL) {
 				crtc->active = true;
 				parse_crtc(tmp, crtc);
 
-				n = ftell(fid);
-				crtc->n_planes = parse_planes(fid, NULL);
+				n = ftell(file);
+				crtc->n_planes = parse_planes(file, NULL);
 				crtc->planes = calloc(crtc->n_planes, sizeof(*crtc->planes));
 				igt_assert_f(crtc->planes, "Failed to allocate memory for %d planes\n", crtc->n_planes);
 
-				fseek(fid, n, SEEK_SET);
-				parse_planes(fid, crtc->planes);
+				fseek(file, n, SEEK_SET);
+				parse_planes(file, crtc->planes);
 
 				if (crtc->pipe != pipe) {
 					free(crtc->planes);
@@ -1365,18 +1481,19 @@ void kmstest_get_crtc(enum pipe pipe, struct kmstest_crtc *crtc)
 		line++;
 	}
 
-	fclose(fid);
+	fclose(file);
+	close(fd);
 
 	igt_skip_on(ncrtc == 0);
 }
 
-void igt_assert_plane_visible(enum pipe pipe, bool visibility)
+void igt_assert_plane_visible(int fd, enum pipe pipe, bool visibility)
 {
 	struct kmstest_crtc crtc;
 	int i;
 	bool visible;
 
-	kmstest_get_crtc(pipe, &crtc);
+	kmstest_get_crtc(fd, pipe, &crtc);
 
 	visible = true;
 	for (i = 0; i < crtc.n_planes; i++) {
@@ -1434,17 +1551,13 @@ static void igt_display_log_shift(igt_display_t *display, int shift)
 	igt_assert(display->log_shift >= 0);
 }
 
-static void igt_output_refresh(igt_output_t *output, bool final)
+static void igt_output_refresh(igt_output_t *output)
 {
 	igt_display_t *display = output->display;
 	unsigned long crtc_idx_mask;
 
 	crtc_idx_mask = output->pending_crtc_idx_mask;
 
-	/* we mask out the pipes already in use */
-	if (final)
-		crtc_idx_mask &= ~display->pipes_in_use;
-
 	kmstest_free_connector_config(&output->config);
 
 	_kmstest_connector_config(display->drm_fd, output->id, crtc_idx_mask,
@@ -1458,10 +1571,15 @@ static void igt_output_refresh(igt_output_t *output, bool final)
 			       -1);
 	}
 
-	if (output->config.connector)
+	if (output->config.connector) {
 		igt_atomic_fill_connector_props(display, output,
 			IGT_NUM_CONNECTOR_PROPS, igt_connector_prop_names);
 
+		kmstest_set_connector_broadcast_rgb(display->drm_fd,
+						    output->config.connector,
+						    BROADCAST_RGB_FULL);
+	}
+
 	if (output->use_override_mode)
 		output->config.default_mode = output->override_mode;
 
@@ -1470,9 +1588,6 @@ static void igt_output_refresh(igt_output_t *output, bool final)
 
 	LOG(display, "%s: Selecting pipe %s\n", output->name,
 	    kmstest_pipe_name(output->config.pipe));
-
-	if (final)
-		display->pipes_in_use |= 1 << output->config.pipe;
 }
 
 static bool
@@ -1679,11 +1794,11 @@ void igt_display_init(igt_display_t *display, int drm_fd)
 		 */
 		igt_assert_eq(pipe->plane_primary, 0);
 
-		/*
-		 * There should be no gaps. If there is, something happened
-		 * which we can't handle (e.g. all planes are cursors).
-		 */
-		igt_assert_eq(p, last_plane);
+		/* Check that we filled every slot exactly once */
+		if (display->has_cursor_plane)
+			igt_assert_eq(p, last_plane);
+		else
+			igt_assert_eq(p, n_planes);
 
 		pipe->n_planes = n_planes;
 
@@ -1714,7 +1829,7 @@ void igt_display_init(igt_display_t *display, int drm_fd)
 		output->id = resources->connectors[i];
 		output->display = display;
 
-		igt_output_refresh(output, false);
+		igt_output_refresh(output);
 
 		output->config.pipe_changed = true;
 	}
@@ -1804,35 +1919,35 @@ void igt_display_fini(igt_display_t *display)
 
 static void igt_display_refresh(igt_display_t *display)
 {
-	int i, j;
+	igt_output_t *output;
+	int i;
 
-	display->pipes_in_use = 0;
+	unsigned long pipes_in_use = 0;
 
        /* Check that two outputs aren't trying to use the same pipe */
-        for (i = 0; i < display->n_outputs; i++) {
-                igt_output_t *a = &display->outputs[i];
-
-                if (!a->pending_crtc_idx_mask)
-                        continue;
-
-                for (j = 0; j < display->n_outputs; j++) {
-                        igt_output_t *b = &display->outputs[j];
-
-                        if (i == j)
-                                continue;
-
-                        igt_assert_f(a->pending_crtc_idx_mask !=
-                                     b->pending_crtc_idx_mask,
-                                     "%s and %s are both trying to use pipe %s\n",
-                                     igt_output_name(a), igt_output_name(b),
-                                     kmstest_pipe_name(ffs(a->pending_crtc_idx_mask) - 1));
-                }
-        }
-
 	for (i = 0; i < display->n_outputs; i++) {
-		igt_output_t *output = &display->outputs[i];
+		output = &display->outputs[i];
 
-		igt_output_refresh(output, true);
+		if (pipes_in_use & output->pending_crtc_idx_mask)
+			goto report_dup;
+
+		pipes_in_use |= output->pending_crtc_idx_mask;
+
+		if (output->force_reprobe)
+			igt_output_refresh(output);
+	}
+
+	return;
+
+report_dup:
+	for (; i > 0; i--) {
+		igt_output_t *b = &display->outputs[i - 1];
+
+		igt_assert_f(output->pending_crtc_idx_mask !=
+			     b->pending_crtc_idx_mask,
+			     "%s and %s are both trying to use pipe %s\n",
+			     igt_output_name(output), igt_output_name(b),
+			     kmstest_pipe_name(ffs(b->pending_crtc_idx_mask) - 1));
 	}
 }
 
@@ -2761,7 +2876,7 @@ void igt_output_set_pipe(igt_output_t *output, enum pipe pipe)
 
 	output->config.pipe_changed = true;
 
-	igt_output_refresh(output, false);
+	igt_output_refresh(output);
 }
 
 void igt_output_set_scaling_mode(igt_output_t *output, uint64_t scaling_mode)
@@ -3020,8 +3135,7 @@ void igt_crtc_set_background(igt_pipe_t *pipe, uint64_t background)
 	pipe->background_changed = true;
 }
 
-
-void igt_wait_for_vblank(int drm_fd, enum pipe pipe)
+void igt_wait_for_vblank_count(int drm_fd, enum pipe pipe, int count)
 {
 	drmVBlank wait_vbl;
 	uint32_t pipe_id_flag;
@@ -3031,11 +3145,16 @@ void igt_wait_for_vblank(int drm_fd, enum pipe pipe)
 
 	wait_vbl.request.type = DRM_VBLANK_RELATIVE;
 	wait_vbl.request.type |= pipe_id_flag;
-	wait_vbl.request.sequence = 1;
+	wait_vbl.request.sequence = count;
 
 	igt_assert(drmWaitVBlank(drm_fd, &wait_vbl) == 0);
 }
 
+void igt_wait_for_vblank(int drm_fd, enum pipe pipe)
+{
+	igt_wait_for_vblank_count(drm_fd, pipe, 1);
+}
+
 /**
  * igt_enable_connectors:
  *
