@@ -51,6 +51,7 @@
 #include "intel_io.h"
 #include "drmtest.h"
 #include "igt_aux.h"
+#include "igt_debugfs.h"
 
 /**
  * intel_get_total_ram_mb:
@@ -96,8 +97,11 @@ intel_get_avail_ram_mb(void)
 
 #ifdef HAVE_STRUCT_SYSINFO_TOTALRAM /* Linux */
 	struct sysinfo sysinf;
+	int fd;
 
-	intel_purge_vm_caches();
+	fd = drm_open_driver(DRIVER_INTEL);
+	intel_purge_vm_caches(fd);
+	close(fd);
 
 	igt_assert(sysinfo(&sysinf) == 0);
 	retval = sysinf.freeram;
@@ -291,10 +295,12 @@ void intel_require_memory(uint64_t count, uint64_t size, unsigned mode)
 	igt_skip_on_simulation();
 }
 
-void intel_purge_vm_caches(void)
+void intel_purge_vm_caches(int drm_fd)
 {
 	int fd;
 
+	igt_drop_caches_set(drm_fd, DROP_SHRINK_ALL);
+
 	fd = open("/proc/sys/vm/drop_caches", O_WRONLY);
 	if (fd >= 0) {
 		/* BIT(2): Be quiet. Cannot be combined with other operations,
