@@ -120,20 +120,6 @@ static void test_invalid_buffer(int in)
 	teardown(fd);
 }
 
-static uint32_t dumb_create(int fd)
-{
-	struct drm_mode_create_dumb arg;
-
-	arg.bpp = 32;
-	arg.width = 32;
-	arg.height = 32;
-
-	do_ioctl(fd, DRM_IOCTL_MODE_CREATE_DUMB, &arg);
-	igt_assert(arg.size >= 4096);
-
-	return arg.handle;
-}
-
 static void test_fault_buffer(int in)
 {
 	int fd = setup(in, 0);
@@ -141,7 +127,7 @@ static void test_fault_buffer(int in)
 	char *buf;
 
 	memset(&arg, 0, sizeof(arg));
-	arg.handle = dumb_create(fd);
+	arg.handle = kmstest_dumb_create(fd, 32, 32, 32, NULL, NULL);
 
 	do_ioctl(fd, DRM_IOCTL_MODE_MAP_DUMB, &arg);
 
@@ -189,46 +175,12 @@ static void test_short_buffer(int in, int nonblock)
 	teardown(fd);
 }
 
-static int pipe0_enabled(int fd)
+static bool crtc0_active(int fd)
 {
-	struct drm_mode_card_res res;
-	uint32_t crtcs[32];
-	int i;
+	union drm_wait_vblank vbl = {};
 
-	/* We assume we can generate events on pipe 0. So we have better
-	 * make sure that is running!
-	 */
-
-	memset(&res, 0, sizeof(res));
-	res.count_crtcs = 32;
-	res.crtc_id_ptr = (uintptr_t)crtcs;
-
-	if (drmIoctl(fd, DRM_IOCTL_MODE_GETRESOURCES, &res))
-		return 0;
-
-	if (res.count_crtcs > 32)
-		return 0;
-
-	for (i = 0; i < res.count_crtcs; i++) {
-		struct drm_i915_get_pipe_from_crtc_id get_pipe;
-		struct drm_mode_crtc mode;
-
-		memset(&get_pipe, 0, sizeof(get_pipe));
-		memset(&mode, 0, sizeof(mode));
-
-		mode.crtc_id = crtcs[i];
-
-		get_pipe.pipe = -1;
-		get_pipe.crtc_id = mode.crtc_id;
-		drmIoctl(fd, DRM_IOCTL_I915_GET_PIPE_FROM_CRTC_ID, &get_pipe);
-		if (get_pipe.pipe)
-			continue;
-
-		drmIoctl(fd, DRM_IOCTL_MODE_GETCRTC, &mode);
-		return mode.mode_valid && mode.mode.clock;
-	}
-
-	return 0;
+	vbl.request.type = DRM_VBLANK_RELATIVE;
+	return drmIoctl(fd, DRM_IOCTL_WAIT_VBLANK, &vbl) == 0;
 }
 
 igt_main
@@ -239,8 +191,8 @@ igt_main
 	siginterrupt(SIGALRM, 1);
 
 	igt_fixture {
-		fd = drm_open_driver_master(DRIVER_INTEL);
-		igt_require(pipe0_enabled(fd));
+		fd = drm_open_driver_master(DRIVER_ANY);
+		igt_require(crtc0_active(fd));
 	}
 
 	igt_subtest("invalid-buffer")
