@@ -117,11 +117,11 @@ static uint32_t busy_blt(int fd)
 	igt_assert(r - reloc <= ARRAY_SIZE(reloc));
 	munmap(map, 4096);
 
-	object[1].relocs_ptr = (uintptr_t)reloc;
+	object[1].relocs_ptr = to_user_pointer(reloc);
 	object[1].relocation_count = r - reloc;
 
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (unsigned long)object;
+	execbuf.buffers_ptr = to_user_pointer(object);
 	execbuf.buffer_count = 2;
 	if (gen >= 6)
 		execbuf.flags = I915_EXEC_BLT;
@@ -149,7 +149,7 @@ static bool exec_noop(int fd,
 	exec[2].handle = handles[BATCH];
 
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)exec;
+	execbuf.buffers_ptr = to_user_pointer(exec);
 	execbuf.buffer_count = 3;
 	execbuf.flags = ring;
 	igt_debug("Queuing handle for %s on ring %d\n",
@@ -236,7 +236,7 @@ static void one(int fd, unsigned ring, uint32_t flags, unsigned test_flags)
 	int i, count, timeout;
 
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)obj;
+	execbuf.buffers_ptr = to_user_pointer(obj);
 	execbuf.buffer_count = 2;
 	execbuf.flags = ring | flags;
 	if (gen < 6)
@@ -246,7 +246,7 @@ static void one(int fd, unsigned ring, uint32_t flags, unsigned test_flags)
 	obj[SCRATCH].handle = gem_create(fd, 4096);
 
 	obj[BATCH].handle = gem_create(fd, size);
-	obj[BATCH].relocs_ptr = (uintptr_t)store;
+	obj[BATCH].relocs_ptr = to_user_pointer(store);
 	obj[BATCH].relocation_count = ARRAY_SIZE(store);
 	memset(store, 0, sizeof(store));
 
@@ -316,10 +316,10 @@ static void one(int fd, unsigned ring, uint32_t flags, unsigned test_flags)
 			if (e->exec_id == 0 || e->exec_id == ring)
 				continue;
 
-			if (e->exec_id == I915_EXEC_BSD && gen == 6)
+			if (!gem_has_ring(fd, e->exec_id | e->flags))
 				continue;
 
-			if (!gem_has_ring(fd, e->exec_id | e->flags))
+			if (!gem_can_store_dword(fd, e->exec_id | e->flags))
 				continue;
 
 			igt_debug("Testing %s in parallel\n", e->name);
@@ -460,137 +460,44 @@ static bool has_semaphores(int fd)
 
 static bool has_extended_busy_ioctl(int fd)
 {
-	const int gen = intel_gen(intel_get_drm_devid(fd));
-	struct drm_i915_gem_exec_object2 obj;
-	struct drm_i915_gem_relocation_entry reloc;
-	struct drm_i915_gem_execbuffer2 execbuf;
+	igt_spin_t *spin = igt_spin_batch_new(fd, I915_EXEC_RENDER, 0);
 	uint32_t read, write;
-	uint32_t *batch;
-	int i;
 
-	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)&obj;
-	execbuf.buffer_count = 1;
-
-	memset(&obj, 0, sizeof(obj));
-	obj.handle = gem_create(fd, 4096);
-
-	obj.relocs_ptr = (uintptr_t)&reloc;
-	obj.relocation_count = 1;
-	memset(&reloc, 0, sizeof(reloc));
-
-	batch = gem_mmap__gtt(fd, obj.handle, 4096, PROT_WRITE);
-	gem_set_domain(fd, obj.handle,
-			I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
-
-	reloc.target_handle = obj.handle; /* recurse */
-	reloc.presumed_offset = 0;
-	reloc.offset = sizeof(uint32_t);
-	reloc.delta = 0;
-	reloc.read_domains = I915_GEM_DOMAIN_COMMAND;
-	reloc.write_domain = 0;
-
-	i = 0;
-	batch[i] = MI_BATCH_BUFFER_START;
-	if (gen >= 8) {
-		batch[i] |= 1 << 8 | 1;
-		batch[++i] = 0;
-		batch[++i] = 0;
-	} else if (gen >= 6) {
-		batch[i] |= 1 << 8;
-		batch[++i] = 0;
-	} else {
-		batch[i] |= 2 << 6;
-		batch[++i] = 0;
-		if (gen < 4) {
-			batch[i] |= 1;
-			reloc.delta = 1;
-		}
-	}
-	i++;
-
-	gem_execbuf(fd, &execbuf);
-	__gem_busy(fd, obj.handle, &read, &write);
-
-	*batch = MI_BATCH_BUFFER_END;
-	__sync_synchronize();
-	munmap(batch, 4096);
-
-	gem_close(fd, obj.handle);
+	__gem_busy(fd, spin->handle, &read, &write);
+	igt_spin_batch_free(fd, spin);
 
 	return read != 0;
 }
 
 static void basic(int fd, unsigned ring, unsigned flags)
 {
-	const int gen = intel_gen(intel_get_drm_devid(fd));
-	struct drm_i915_gem_exec_object2 obj;
-	struct drm_i915_gem_relocation_entry reloc;
-	struct drm_i915_gem_execbuffer2 execbuf;
+	igt_spin_t *spin = igt_spin_batch_new(fd, ring, 0);
 	struct timespec tv;
-	uint32_t *batch;
-	int i, timeout;
+	int timeout;
 	bool busy;
 
-	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)&obj;
-	execbuf.buffer_count = 1;
-	execbuf.flags = ring;
-
-	memset(&obj, 0, sizeof(obj));
-	obj.handle = gem_create(fd, 4096);
-
-	obj.relocs_ptr = (uintptr_t)&reloc;
-	obj.relocation_count = 1;
-	memset(&reloc, 0, sizeof(reloc));
-
-	batch = gem_mmap__gtt(fd, obj.handle, 4096, PROT_WRITE);
-	gem_set_domain(fd, obj.handle,
-			I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
-
-	reloc.target_handle = obj.handle; /* recurse */
-	reloc.presumed_offset = 0;
-	reloc.offset = sizeof(uint32_t);
-	reloc.delta = 0;
-	reloc.read_domains = I915_GEM_DOMAIN_COMMAND;
-	reloc.write_domain = 0;
-
-	i = 0;
-	batch[i] = MI_BATCH_BUFFER_START;
-	if (gen >= 8) {
-		batch[i] |= 1 << 8 | 1;
-		batch[++i] = 0;
-		batch[++i] = 0;
-	} else if (gen >= 6) {
-		batch[i] |= 1 << 8;
-		batch[++i] = 0;
-	} else {
-		batch[i] |= 2 << 6;
-		batch[++i] = 0;
-		if (gen < 4) {
-			batch[i] |= 1;
-			reloc.delta = 1;
-		}
-	}
-	i++;
-
-	gem_execbuf(fd, &execbuf);
-	busy = gem_bo_busy(fd, obj.handle);
+	busy = gem_bo_busy(fd, spin->handle);
 
 	timeout = 120;
 	if ((flags & HANG) == 0) {
-		*batch = MI_BATCH_BUFFER_END;
-		__sync_synchronize();
+		igt_spin_batch_end(spin);
 		timeout = 1;
 	}
-	munmap(batch, 4096);
 
 	igt_assert(busy);
 	memset(&tv, 0, sizeof(tv));
-	while (gem_bo_busy(fd, obj.handle))
-		igt_assert(igt_seconds_elapsed(&tv) < timeout);
+	while (gem_bo_busy(fd, spin->handle)) {
+		if (igt_seconds_elapsed(&tv) > timeout) {
+			igt_debugfs_dump(fd, "i915_engine_info");
+			igt_debugfs_dump(fd, "i915_hangcheck_info");
+			igt_assert_f(igt_seconds_elapsed(&tv) < timeout,
+				     "%s batch did not complete within %ds\n",
+				     flags & HANG ? "Hanging" : "Normal",
+				     timeout);
+		}
+	}
 
-	gem_close(fd, obj.handle);
+	igt_spin_batch_free(fd, spin);
 }
 
 igt_main
@@ -602,6 +509,8 @@ igt_main
 
 	igt_fixture {
 		fd = drm_open_driver_master(DRIVER_INTEL);
+		igt_require_gem(fd);
+		igt_require(gem_can_store_dword(fd, 0));
 	}
 
 	igt_subtest_group {
@@ -622,12 +531,9 @@ igt_main
 		}
 
 		igt_subtest_group {
-			int gen = 0;
-
 			igt_fixture {
 				igt_require(has_extended_busy_ioctl(fd));
 				gem_require_mmap_wc(fd);
-				gen = intel_gen(intel_get_drm_devid(fd));
 			}
 
 			for (e = intel_execution_engines; e->name; e++) {
@@ -637,9 +543,7 @@ igt_main
 
 				igt_subtest_f("extended-%s", e->name) {
 					gem_require_ring(fd, e->exec_id | e->flags);
-					igt_skip_on_f(gen == 6 &&
-						      e->exec_id == I915_EXEC_BSD,
-						      "MI_STORE_DATA broken on gen6 bsd\n");
+					igt_require(gem_can_store_dword(fd, e->exec_id | e->flags));
 					gem_quiescent_gpu(fd);
 					one(fd, e->exec_id, e->flags, 0);
 					gem_quiescent_gpu(fd);
@@ -653,9 +557,8 @@ igt_main
 
 				igt_subtest_f("extended-parallel-%s", e->name) {
 					gem_require_ring(fd, e->exec_id | e->flags);
-					igt_skip_on_f(gen == 6 &&
-						      e->exec_id == I915_EXEC_BSD,
-						      "MI_STORE_DATA broken on gen6 bsd\n");
+					igt_require(gem_can_store_dword(fd, e->exec_id | e->flags));
+
 					gem_quiescent_gpu(fd);
 					one(fd, e->exec_id, e->flags, PARALLEL);
 					gem_quiescent_gpu(fd);
@@ -705,12 +608,9 @@ igt_main
 		}
 
 		igt_subtest_group {
-			int gen = 0;
-
 			igt_fixture {
 				igt_require(has_extended_busy_ioctl(fd));
 				gem_require_mmap_wc(fd);
-				gen = intel_gen(intel_get_drm_devid(fd));
 			}
 
 			for (e = intel_execution_engines; e->name; e++) {
@@ -720,9 +620,8 @@ igt_main
 
 				igt_subtest_f("extended-hang-%s", e->name) {
 					gem_require_ring(fd, e->exec_id | e->flags);
-					igt_skip_on_f(gen == 6 &&
-						      e->exec_id == I915_EXEC_BSD,
-						      "MI_STORE_DATA broken on gen6 bsd\n");
+					igt_require(gem_can_store_dword(fd, e->exec_id | e->flags));
+
 					gem_quiescent_gpu(fd);
 					one(fd, e->exec_id, e->flags, HANG);
 					gem_quiescent_gpu(fd);
