@@ -243,7 +243,7 @@ static void create_userptr_require(const struct create *create, unsigned count)
 			has_userptr = drmIoctl(fd,
 					 LOCAL_IOCTL_I915_GEM_USERPTR,
 					 &arg) == 0;
-			free((void *)(uintptr_t)arg.user_ptr);
+			free(from_user_pointer(arg.user_ptr));
 		}
 
 	}
@@ -263,7 +263,7 @@ userptr_create_bo(const struct buffers *b)
 	ptr = mmap(NULL, userptr.user_size,
 		   PROT_READ | PROT_WRITE, MAP_ANON | MAP_SHARED, -1, 0);
 	igt_assert(ptr != (void *)-1);
-	userptr.user_ptr = (uintptr_t)ptr;
+	userptr.user_ptr = to_user_pointer(ptr);
 
 #if 0
 	do_or_die(drmIoctl(fd, LOCAL_IOCTL_I915_GEM_USERPTR, &userptr));
@@ -275,7 +275,7 @@ userptr_create_bo(const struct buffers *b)
 					userptr.user_size, 0);
 	igt_assert(bo);
 #endif
-	bo->virtual = (void *)(uintptr_t)userptr.user_ptr;
+	bo->virtual = from_user_pointer(userptr.user_ptr);
 
 	return bo;
 }
@@ -671,9 +671,9 @@ gpu_set_bo(struct buffers *buffers, drm_intel_bo *bo, uint32_t val)
 
 	gem_exec[1].handle = gem_create(fd, 4096);
 	gem_exec[1].relocation_count = 1;
-	gem_exec[1].relocs_ptr = (uintptr_t)reloc;
+	gem_exec[1].relocs_ptr = to_user_pointer(reloc);
 
-	execbuf.buffers_ptr = (uintptr_t)gem_exec;
+	execbuf.buffers_ptr = to_user_pointer(gem_exec);
 	execbuf.buffer_count = 2;
 	execbuf.batch_len = (b - buf) * sizeof(buf[0]);
 	if (gen >= 6)
@@ -917,8 +917,8 @@ static void wc_copy_bo(struct buffers *b, drm_intel_bo *dst, drm_intel_bo *src)
 	const int size = b->page_size;
 	void *d, *s;
 
-	gem_set_domain(fd, src->handle, I915_GEM_DOMAIN_GTT, 0);
-	gem_set_domain(fd, dst->handle, I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
+	gem_set_domain(fd, src->handle, I915_GEM_DOMAIN_WC, 0);
+	gem_set_domain(fd, dst->handle, I915_GEM_DOMAIN_WC, I915_GEM_DOMAIN_WC);
 
 	s = gem_mmap__wc(fd, src->handle, 0, size, PROT_READ);
 	d = gem_mmap__wc(fd, dst->handle, 0, size, PROT_WRITE);
@@ -957,7 +957,7 @@ static igt_hang_t all_hang(void)
 	gem_write(fd, obj.handle, 0, &bbe, sizeof(&bbe));
 
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)&obj;
+	execbuf.buffers_ptr = to_user_pointer(&obj);
 	execbuf.buffer_count = 1;
 
 	for_each_engine(fd, engine) {
@@ -1833,6 +1833,7 @@ igt_main
 		allow_unlimited_files();
 
 		fd = drm_open_driver(DRIVER_INTEL);
+		igt_require_gem(fd);
 		intel_detect_and_clear_missed_interrupts(fd);
 		devid = intel_get_drm_devid(fd);
 		gen = intel_gen(devid);
@@ -1897,6 +1898,21 @@ igt_main
 				run_modes(name, c, modes, s, count);
 			}
 
+			snprintf(name, sizeof(name), "%s%s-%s",
+				 c->name, s->name, "shrink");
+			igt_subtest_group {
+				igt_fixture {
+					count = num_buffers(gem_mappable_aperture_size(),
+							    s, c, CHECK_RAM);
+
+					igt_fork_shrink_helper(fd);
+				}
+				run_modes(name, c, modes, s, count);
+
+				igt_fixture
+					igt_stop_shrink_helper();
+			}
+
 			/* Use the entire mappable aperture, force swapping */
 			snprintf(name, sizeof(name), "%s%s-%s",
 				 c->name, s->name, "swap");
