@@ -22,6 +22,7 @@
  */
 
 #include "igt.h"
+#include "igt_dummyload.h"
 
 IGT_TEST_DESCRIPTION("Basic sanity check of execbuf-ioctl relocations.");
 
@@ -86,10 +87,10 @@ static void write_dword(int fd,
 	reloc.write_domain = I915_GEM_DOMAIN_INSTRUCTION;
 
 	obj[1].relocation_count = 1;
-	obj[1].relocs_ptr = (uintptr_t)&reloc;
+	obj[1].relocs_ptr = to_user_pointer(&reloc);
 
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)obj;
+	execbuf.buffers_ptr = to_user_pointer(obj);
 	execbuf.buffer_count = 2;
 	execbuf.flags = I915_EXEC_SECURE;
 	gem_execbuf(fd, &execbuf);
@@ -109,7 +110,10 @@ static void from_mmap(int fd, uint64_t size, enum mode mode)
 	uint64_t max, i;
 	int retry = 2;
 
-	intel_require_memory(1, size, CHECK_RAM);
+	/* Worst case is that the kernel has to copy the entire incoming
+	 * reloc[], so double the memory requirements.
+	 */
+	intel_require_memory(2, size, CHECK_RAM);
 
 	memset(&obj, 0, sizeof(obj));
 	obj.handle = gem_create(fd, 4096);
@@ -141,7 +145,7 @@ static void from_mmap(int fd, uint64_t size, enum mode mode)
 		reloc_handle = gem_create(fd, size);
 		relocs = gem_mmap__wc(fd, reloc_handle, 0, size, PROT_WRITE);
 		gem_set_domain(fd, reloc_handle,
-			       I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
+			       I915_GEM_DOMAIN_WC, I915_GEM_DOMAIN_WC);
 		gem_close(fd, reloc_handle);
 		break;
 	}
@@ -155,13 +159,13 @@ static void from_mmap(int fd, uint64_t size, enum mode mode)
 		relocs[i].write_domain = 0;
 	}
 	obj.relocation_count = max;
-	obj.relocs_ptr = (uintptr_t)relocs;
+	obj.relocs_ptr = to_user_pointer(relocs);
 
 	if (mode & RO)
 		mprotect(relocs, size, PROT_READ);
 
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)&obj;
+	execbuf.buffers_ptr = to_user_pointer(&obj);
 	execbuf.buffer_count = 1;
 	while (relocs[0].presumed_offset == ~0ull && retry--)
 		gem_execbuf(fd, &execbuf);
@@ -186,6 +190,8 @@ static void from_gpu(int fd)
 	uint32_t reloc_handle;
 	uint64_t value;
 
+	igt_require(gem_can_store_dword(fd, 0));
+
 	memset(&obj, 0, sizeof(obj));
 	obj.handle = gem_create(fd, 4096);
 	gem_write(fd, obj.handle, 0, &bbe, sizeof(bbe));
@@ -213,10 +219,10 @@ static void from_gpu(int fd)
 	gem_close(fd, reloc_handle);
 
 	obj.relocation_count = 1;
-	obj.relocs_ptr = (uintptr_t)relocs;
+	obj.relocs_ptr = to_user_pointer(relocs);
 
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)&obj;
+	execbuf.buffers_ptr = to_user_pointer(&obj);
 	execbuf.buffer_count = 1;
 	gem_execbuf(fd, &execbuf);
 	gem_read(fd, obj.handle, 1024, &value, sizeof(value));
@@ -227,11 +233,6 @@ static void from_gpu(int fd)
 	munmap(relocs, 4096);
 }
 
-static bool ignore_engine(int gen, unsigned engine)
-{
-	return gen == 6 && (engine & ~(3<<13)) == I915_EXEC_BSD;
-}
-
 static void check_bo(int fd, uint32_t handle)
 {
 	uint32_t *map;
@@ -258,12 +259,12 @@ static void active(int fd, unsigned engine)
 	nengine = 0;
 	if (engine == -1) {
 		for_each_engine(fd, engine) {
-			if (!ignore_engine(gen, engine))
+			if (gem_can_store_dword(fd, engine))
 				engines[nengine++] = engine;
 		}
 	} else {
 		igt_require(gem_has_ring(fd, engine));
-		igt_require(!ignore_engine(gen, engine));
+		igt_require(gem_can_store_dword(fd, engine));
 		engines[nengine++] = engine;
 	}
 	igt_require(nengine);
@@ -271,7 +272,7 @@ static void active(int fd, unsigned engine)
 	memset(obj, 0, sizeof(obj));
 	obj[0].handle = gem_create(fd, 4096);
 	obj[1].handle = gem_create(fd, 64*1024);
-	obj[1].relocs_ptr = (uintptr_t)&reloc;
+	obj[1].relocs_ptr = to_user_pointer(&reloc);
 	obj[1].relocation_count = 1;
 
 	memset(&reloc, 0, sizeof(reloc));
@@ -283,7 +284,7 @@ static void active(int fd, unsigned engine)
 	reloc.write_domain = I915_GEM_DOMAIN_INSTRUCTION;
 
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)obj;
+	execbuf.buffers_ptr = to_user_pointer(obj);
 	execbuf.buffer_count = 2;
 	if (gen < 6)
 		execbuf.flags |= I915_EXEC_SECURE;
@@ -323,209 +324,294 @@ static void active(int fd, unsigned engine)
 	gem_close(fd, obj[0].handle);
 }
 
-static bool has_64bit_reloc(int fd)
+static bool has_64b_reloc(int fd)
 {
 	return intel_gen(intel_get_drm_devid(fd)) >= 8;
 }
 
-static void basic_cpu(int fd)
+#define NORELOC 1
+#define ACTIVE 2
+#define HANG 4
+static void basic_reloc(int fd, unsigned before, unsigned after, unsigned flags)
 {
+#define OBJSZ 8192
 	struct drm_i915_gem_relocation_entry reloc;
 	struct drm_i915_gem_exec_object2 obj;
 	struct drm_i915_gem_execbuffer2 execbuf;
-	uint32_t bbe = MI_BATCH_BUFFER_END;
-	uint32_t trash;
-	uint64_t offset;
-	char *wc;
+	uint64_t address_mask = has_64b_reloc(fd) ? ~(uint64_t)0 : ~(uint32_t)0;
+	const uint32_t bbe = MI_BATCH_BUFFER_END;
+	unsigned int reloc_offset;
 
 	memset(&obj, 0, sizeof(obj));
-
-	obj.handle = gem_create(fd, 4096);
-	obj.relocs_ptr = (uintptr_t)&reloc;
+	obj.handle = gem_create(fd, OBJSZ);
+	obj.relocs_ptr = to_user_pointer(&reloc);
 	obj.relocation_count = 1;
 	gem_write(fd, obj.handle, 0, &bbe, sizeof(bbe));
 
-	memset(&reloc, 0, sizeof(reloc));
-	reloc.offset = 4000;
-	reloc.target_handle = obj.handle;
-	reloc.read_domains = I915_GEM_DOMAIN_INSTRUCTION;
-
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)&obj;
+	execbuf.buffers_ptr = to_user_pointer(&obj);
 	execbuf.buffer_count = 1;
+	if (flags & NORELOC)
+		execbuf.flags |= LOCAL_I915_EXEC_NO_RELOC;
 
-	wc = gem_mmap__wc(fd, obj.handle, 0, 4096, PROT_WRITE);
-	offset = -1;
-	memcpy(wc + 4000, &offset, sizeof(offset));
+	for (reloc_offset = 4096 - 8; reloc_offset <= 4096 + 8; reloc_offset += 4) {
+		igt_spin_t *spin = NULL;
+		uint32_t trash = 0;
+		uint64_t offset;
 
-	gem_set_domain(fd, obj.handle,
-		       I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_CPU);
-	gem_execbuf(fd, &execbuf);
+		obj.offset = -1;
 
-	offset = 0;
-	memcpy(&offset, wc + 4000, has_64bit_reloc(fd) ? 8 : 4);
-	munmap(wc, 4096);
+		memset(&reloc, 0, sizeof(reloc));
+		reloc.offset = reloc_offset;
+		reloc.target_handle = obj.handle;
+		reloc.read_domains = I915_GEM_DOMAIN_INSTRUCTION;
+		reloc.presumed_offset = -1;
 
-	if (reloc.presumed_offset == -1)
-		igt_warn("reloc.presumed_offset == -1\n");
-	else
-		igt_assert_eq_u64(reloc.presumed_offset, offset);
-	igt_assert_eq_u64(obj.offset, offset);
+		if (before) {
+			char *wc;
 
-	/* Simulate relocation */
-	trash = obj.handle;
-	obj.handle = gem_create(fd, 4096);
-	gem_write(fd, obj.handle, 0, &bbe, sizeof(bbe));
-	reloc.target_handle = obj.handle;
+			if (before == I915_GEM_DOMAIN_CPU)
+				wc = gem_mmap__cpu(fd, obj.handle, 0, OBJSZ, PROT_WRITE);
+			else if (before == I915_GEM_DOMAIN_GTT)
+				wc = gem_mmap__gtt(fd, obj.handle, OBJSZ, PROT_WRITE);
+			else if (before == I915_GEM_DOMAIN_WC)
+				wc = gem_mmap__wc(fd, obj.handle, 0, OBJSZ, PROT_WRITE);
+			else
+				igt_assert(0);
+			gem_set_domain(fd, obj.handle, before, before);
+			offset = -1;
+			memcpy(wc + reloc_offset, &offset, sizeof(offset));
+			munmap(wc, OBJSZ);
+		} else {
+			offset = -1;
+			gem_write(fd, obj.handle, reloc_offset, &offset, sizeof(offset));
+		}
 
-	wc = gem_mmap__wc(fd, obj.handle, 0, 4096, PROT_WRITE);
-	offset = -1;
-	memcpy(wc + 4000, &offset, sizeof(offset));
+		if (flags & ACTIVE) {
+			spin = igt_spin_batch_new(fd, I915_EXEC_DEFAULT, obj.handle);
+			if (!(flags & HANG))
+				igt_spin_batch_set_timeout(spin, NSEC_PER_SEC/100);
+			igt_assert(gem_bo_busy(fd, obj.handle));
+		}
 
-	gem_set_domain(fd, obj.handle,
-		       I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_CPU);
-	gem_execbuf(fd, &execbuf);
+		gem_execbuf(fd, &execbuf);
 
-	offset = 0;
-	memcpy(&offset, wc + 4000, has_64bit_reloc(fd) ? 8 : 4);
-	munmap(wc, 4096);
+		if (after) {
+			char *wc;
 
-	if (reloc.presumed_offset == -1)
-		igt_warn("reloc.presumed_offset == -1\n");
-	else
-		igt_assert_eq_u64(reloc.presumed_offset, offset);
-	igt_assert_eq_u64(obj.offset, offset);
+			if (after == I915_GEM_DOMAIN_CPU)
+				wc = gem_mmap__cpu(fd, obj.handle, 0, OBJSZ, PROT_READ);
+			else if (after == I915_GEM_DOMAIN_GTT)
+				wc = gem_mmap__gtt(fd, obj.handle, OBJSZ, PROT_READ);
+			else if (after == I915_GEM_DOMAIN_WC)
+				wc = gem_mmap__wc(fd, obj.handle, 0, OBJSZ, PROT_READ);
+			else
+				igt_assert(0);
+			gem_set_domain(fd, obj.handle, after, 0);
+			offset = ~reloc.presumed_offset & address_mask;
+			memcpy(&offset, wc + reloc_offset, has_64b_reloc(fd) ? 8 : 4);
+			munmap(wc, OBJSZ);
+		} else {
+			offset = ~reloc.presumed_offset & address_mask;
+			gem_read(fd, obj.handle, reloc_offset, &offset, has_64b_reloc(fd) ? 8 : 4);
+		}
 
-	gem_close(fd, obj.handle);
-	gem_close(fd, trash);
-}
+		if (reloc.presumed_offset == -1)
+			igt_warn("reloc.presumed_offset == -1\n");
+		else
+			igt_assert_eq_u64(reloc.presumed_offset, offset);
+		igt_assert_eq_u64(obj.offset, offset);
 
-static void basic_gtt(int fd)
-{
-	struct drm_i915_gem_relocation_entry reloc;
-	struct drm_i915_gem_exec_object2 obj;
-	struct drm_i915_gem_execbuffer2 execbuf;
-	uint32_t bbe = MI_BATCH_BUFFER_END;
-	uint64_t offset;
-	char *wc;
+		igt_spin_batch_free(fd, spin);
 
-	memset(&obj, 0, sizeof(obj));
+		/* Simulate relocation */
+		if (flags & NORELOC) {
+			obj.offset += OBJSZ;
+			reloc.presumed_offset += OBJSZ;
+		} else {
+			trash = obj.handle;
+			obj.handle = gem_create(fd, OBJSZ);
+			gem_write(fd, obj.handle, 0, &bbe, sizeof(bbe));
+			reloc.target_handle = obj.handle;
+		}
 
-	obj.handle = gem_create(fd, 4096);
-	obj.relocs_ptr = (uintptr_t)&reloc;
-	obj.relocation_count = 1;
-	gem_write(fd, obj.handle, 0, &bbe, sizeof(bbe));
+		if (before) {
+			char *wc;
 
-	memset(&reloc, 0, sizeof(reloc));
-	reloc.offset = 4000;
-	reloc.target_handle = obj.handle;
-	reloc.read_domains = I915_GEM_DOMAIN_INSTRUCTION;
+			if (before == I915_GEM_DOMAIN_CPU)
+				wc = gem_mmap__cpu(fd, obj.handle, 0, OBJSZ, PROT_WRITE);
+			else if (before == I915_GEM_DOMAIN_GTT)
+				wc = gem_mmap__gtt(fd, obj.handle, OBJSZ, PROT_WRITE);
+			else if (before == I915_GEM_DOMAIN_WC)
+				wc = gem_mmap__wc(fd, obj.handle, 0, OBJSZ, PROT_WRITE);
+			else
+				igt_assert(0);
+			gem_set_domain(fd, obj.handle, before, before);
+			memcpy(wc + reloc_offset, &reloc.presumed_offset, sizeof(reloc.presumed_offset));
+			munmap(wc, OBJSZ);
+		} else {
+			gem_write(fd, obj.handle, reloc_offset, &reloc.presumed_offset, sizeof(reloc.presumed_offset));
+		}
 
-	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)&obj;
-	execbuf.buffer_count = 1;
+		if (flags & ACTIVE) {
+			spin = igt_spin_batch_new(fd, I915_EXEC_DEFAULT, obj.handle);
+			if (!(flags & HANG))
+				igt_spin_batch_set_timeout(spin, NSEC_PER_SEC/100);
+			igt_assert(gem_bo_busy(fd, obj.handle));
+		}
 
-	wc = gem_mmap__wc(fd, obj.handle, 0, 4096, PROT_WRITE);
-	offset = -1;
-	memcpy(wc + 4000, &offset, sizeof(offset));
+		gem_execbuf(fd, &execbuf);
 
-	gem_set_domain(fd, obj.handle,
-		       I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_CPU);
-	gem_execbuf(fd, &execbuf);
+		if (after) {
+			char *wc;
 
-	offset = 0;
-	memcpy(&offset, wc + 4000, has_64bit_reloc(fd) ? 8 : 4);
+			if (after == I915_GEM_DOMAIN_CPU)
+				wc = gem_mmap__cpu(fd, obj.handle, 0, OBJSZ, PROT_READ);
+			else if (after == I915_GEM_DOMAIN_GTT)
+				wc = gem_mmap__gtt(fd, obj.handle, OBJSZ, PROT_READ);
+			else if (after == I915_GEM_DOMAIN_WC)
+				wc = gem_mmap__wc(fd, obj.handle, 0, OBJSZ, PROT_READ);
+			else
+				igt_assert(0);
+			gem_set_domain(fd, obj.handle, after, 0);
+			offset = ~reloc.presumed_offset & address_mask;
+			memcpy(&offset, wc + reloc_offset, has_64b_reloc(fd) ? 8 : 4);
+			munmap(wc, OBJSZ);
+		} else {
+			offset = ~reloc.presumed_offset & address_mask;
+			gem_read(fd, obj.handle, reloc_offset, &offset, has_64b_reloc(fd) ? 8 : 4);
+		}
 
-	if (reloc.presumed_offset == -1)
-		igt_warn("reloc.presumed_offset == -1\n");
-	else
-		igt_assert_eq_u64(reloc.presumed_offset, offset);
-	igt_assert_eq_u64(obj.offset, offset);
+		if (reloc.presumed_offset == -1)
+			igt_warn("reloc.presumed_offset == -1\n");
+		else
+			igt_assert_eq_u64(reloc.presumed_offset, offset);
+		igt_assert_eq_u64(obj.offset, offset);
 
-	offset = -1;
-	memcpy(wc + 4000, &offset, sizeof(offset));
-
-	/* Simulate relocation */
-	obj.offset += 4096;
-	reloc.presumed_offset += 4096;
-	memcpy(wc + 4000, &obj.offset, has_64bit_reloc(fd) ? 8 : 4);
-
-	gem_execbuf(fd, &execbuf);
-
-	offset = 0;
-	memcpy(&offset, wc + 4000, has_64bit_reloc(fd) ? 8 : 4);
-	munmap(wc, 4096);
-
-	if (reloc.presumed_offset == -1)
-		igt_warn("reloc.presumed_offset == -1\n");
-	else
-		igt_assert_eq_u64(reloc.presumed_offset, offset);
-	igt_assert_eq_u64(obj.offset, offset);
+		igt_spin_batch_free(fd, spin);
+		if (trash)
+			gem_close(fd, trash);
+	}
 
 	gem_close(fd, obj.handle);
 }
 
-static void basic_noreloc(int fd)
+static inline uint64_t sign_extend(uint64_t x, int index)
 {
-	struct drm_i915_gem_relocation_entry reloc;
-	struct drm_i915_gem_exec_object2 obj;
+	int shift = 63 - index;
+	return (int64_t)(x << shift) >> shift;
+}
+
+static uint64_t gen8_canonical_address(uint64_t address)
+{
+	return sign_extend(address, 47);
+}
+
+static void basic_range(int fd, unsigned flags)
+{
+	struct drm_i915_gem_relocation_entry reloc[128];
+	struct drm_i915_gem_exec_object2 obj[128];
 	struct drm_i915_gem_execbuffer2 execbuf;
-	uint32_t bbe = MI_BATCH_BUFFER_END;
-	uint64_t offset;
-	char *wc;
+	uint64_t address_mask = has_64b_reloc(fd) ? ~(uint64_t)0 : ~(uint32_t)0;
+	uint64_t gtt_size = gem_aperture_size(fd);
+	const uint32_t bbe = MI_BATCH_BUFFER_END;
+	igt_spin_t *spin = NULL;
+	int count, n;
 
-	memset(&obj, 0, sizeof(obj));
+	igt_require(gem_has_softpin(fd));
 
-	obj.handle = gem_create(fd, 4096);
-	obj.relocs_ptr = (uintptr_t)&reloc;
-	obj.relocation_count = 1;
-	gem_write(fd, obj.handle, 0, &bbe, sizeof(bbe));
+	for (count = 12; gtt_size >> (count + 1); count++)
+		;
 
-	memset(&reloc, 0, sizeof(reloc));
-	reloc.offset = 4000;
-	reloc.target_handle = obj.handle;
-	reloc.read_domains = I915_GEM_DOMAIN_INSTRUCTION;
+	count -= 12;
 
+	memset(obj, 0, sizeof(obj));
+	memset(reloc, 0, sizeof(reloc));
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)&obj;
-	execbuf.buffer_count = 1;
-	execbuf.flags = LOCAL_I915_EXEC_NO_RELOC;
 
-	wc = gem_mmap__wc(fd, obj.handle, 0, 4096, PROT_WRITE);
-	offset = -1;
-	memcpy(wc + 4000, &offset, sizeof(offset));
+	n = 0;
+	for (int i = 0; i <= count; i++) {
+		obj[n].handle = gem_create(fd, 4096);
+		obj[n].offset = (1ull << (i + 12)) - 4096;
+		obj[n].offset = gen8_canonical_address(obj[n].offset);
+		obj[n].flags = EXEC_OBJECT_PINNED | EXEC_OBJECT_SUPPORTS_48B_ADDRESS;
+		gem_write(fd, obj[n].handle, 0, &bbe, sizeof(bbe));
+		execbuf.buffers_ptr = to_user_pointer(&obj[n]);
+		execbuf.buffer_count = 1;
+		if (__gem_execbuf(fd, &execbuf))
+			continue;
 
-	gem_set_domain(fd, obj.handle,
-		       I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_CPU);
-	gem_execbuf(fd, &execbuf);
+		igt_debug("obj[%d] handle=%d, address=%llx\n",
+			  n, obj[n].handle, (long long)obj[n].offset);
 
-	offset = 0;
-	memcpy(&offset, wc + 4000, has_64bit_reloc(fd) ? 8 : 4);
+		reloc[n].offset = 8 * (n + 1);
+		reloc[n].target_handle = obj[n].handle;
+		reloc[n].read_domains = I915_GEM_DOMAIN_INSTRUCTION;
+		reloc[n].presumed_offset = -1;
+		n++;
+	}
+	for (int i = 1; i < count; i++) {
+		obj[n].handle = gem_create(fd, 4096);
+		obj[n].offset = 1ull << (i + 12);
+		obj[n].offset = gen8_canonical_address(obj[n].offset);
+		obj[n].flags = EXEC_OBJECT_PINNED | EXEC_OBJECT_SUPPORTS_48B_ADDRESS;
+		gem_write(fd, obj[n].handle, 0, &bbe, sizeof(bbe));
+		execbuf.buffers_ptr = to_user_pointer(&obj[n]);
+		execbuf.buffer_count = 1;
+		if (__gem_execbuf(fd, &execbuf))
+			continue;
 
-	if (reloc.presumed_offset == -1)
-		igt_warn("reloc.presumed_offset == -1\n");
-	else
-		igt_assert_eq_u64(reloc.presumed_offset, offset);
-	igt_assert_eq_u64(obj.offset, offset);
+		igt_debug("obj[%d] handle=%d, address=%llx\n",
+			  n, obj[n].handle, (long long)obj[n].offset);
 
-	/* Simulate relocation */
-	obj.offset += 4096;
-	reloc.presumed_offset += 4096;
-	memcpy(wc + 4000, &obj.offset, sizeof(obj.offset));
+		reloc[n].offset = 8 * (n + 1);
+		reloc[n].target_handle = obj[n].handle;
+		reloc[n].read_domains = I915_GEM_DOMAIN_INSTRUCTION;
+		reloc[n].presumed_offset = -1;
+		n++;
+	}
+	igt_require(n);
+
+	obj[n].handle = gem_create(fd, 4096);
+	obj[n].relocs_ptr = to_user_pointer(reloc);
+	obj[n].relocation_count = n;
+	gem_write(fd, obj[n].handle, 0, &bbe, sizeof(bbe));
+
+	execbuf.buffers_ptr = to_user_pointer(obj);
+	execbuf.buffer_count = n + 1;
+
+	if (flags & ACTIVE) {
+		spin = igt_spin_batch_new(fd, 0, obj[n].handle);
+		if (!(flags & HANG))
+			igt_spin_batch_set_timeout(spin, NSEC_PER_SEC/100);
+		igt_assert(gem_bo_busy(fd, obj[n].handle));
+	}
 
 	gem_execbuf(fd, &execbuf);
+	igt_spin_batch_free(fd, spin);
 
-	offset = 0;
-	memcpy(&offset, wc + 4000, has_64bit_reloc(fd) ? 8 : 4);
-	munmap(wc, 4096);
+	for (int i = 0; i < n; i++) {
+		uint64_t offset;
 
-	if (reloc.presumed_offset == -1)
-		igt_warn("reloc.presumed_offset == -1\n");
-	else
-		igt_assert_eq_u64(reloc.presumed_offset, offset);
-	igt_assert_eq_u64(obj.offset, offset);
+		offset = ~reloc[i].presumed_offset & address_mask;
+		gem_read(fd, obj[n].handle, reloc[i].offset,
+			 &offset, has_64b_reloc(fd) ? 8 : 4);
 
-	gem_close(fd, obj.handle);
+		igt_debug("obj[%d] handle=%d, offset=%llx, found=%llx, presumed=%llx\n",
+			  i, obj[i].handle,
+			  (long long)obj[i].offset,
+			  (long long)offset,
+			  (long long)reloc[i].presumed_offset);
+
+		igt_assert_eq_u64(obj[i].offset, offset);
+		if (reloc[i].presumed_offset == -1)
+			igt_warn("reloc.presumed_offset == -1\n");
+		else
+			igt_assert_eq_u64(reloc[i].presumed_offset, offset);
+	}
+
+	for (int i = 0; i <= n; i++)
+		gem_close(fd, obj[i].handle);
 }
 
 static void basic_softpin(int fd)
@@ -542,7 +628,7 @@ static void basic_softpin(int fd)
 	gem_write(fd, obj[1].handle, 0, &bbe, sizeof(bbe));
 
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)&obj[1];
+	execbuf.buffers_ptr = to_user_pointer(&obj[1]);
 	execbuf.buffer_count = 1;
 	gem_execbuf(fd, &execbuf);
 
@@ -552,7 +638,7 @@ static void basic_softpin(int fd)
 	obj[0].offset = obj[1].offset;
 	obj[0].flags = EXEC_OBJECT_PINNED;
 
-	execbuf.buffers_ptr = (uintptr_t)&obj[0];
+	execbuf.buffers_ptr = to_user_pointer(&obj[0]);
 	execbuf.buffer_count = 2;
 
 	gem_execbuf(fd, &execbuf);
@@ -564,20 +650,79 @@ static void basic_softpin(int fd)
 
 igt_main
 {
+	const struct mode {
+		const char *name;
+		unsigned before, after;
+	} modes[] = {
+		{ "cpu", I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_CPU },
+		{ "gtt", I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT },
+		{ "wc", I915_GEM_DOMAIN_WC, I915_GEM_DOMAIN_WC },
+		{ "cpu-gtt", I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_GTT },
+		{ "gtt-cpu", I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_CPU },
+		{ "cpu-wc", I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_WC },
+		{ "wc-cpu", I915_GEM_DOMAIN_WC, I915_GEM_DOMAIN_CPU },
+		{ "gtt-wc", I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_WC },
+		{ "wc-gtt", I915_GEM_DOMAIN_WC, I915_GEM_DOMAIN_GTT },
+		{ "cpu-read", I915_GEM_DOMAIN_CPU, 0 },
+		{ "gtt-read", I915_GEM_DOMAIN_GTT, 0 },
+		{ "wc-read", I915_GEM_DOMAIN_WC, 0 },
+		{ "write-cpu", 0, I915_GEM_DOMAIN_CPU },
+		{ "write-gtt", 0, I915_GEM_DOMAIN_GTT },
+		{ "write-wc", 0, I915_GEM_DOMAIN_WC },
+		{ "write-read", 0, 0 },
+		{ },
+	}, *m;
+	const struct flags {
+		const char *name;
+		unsigned flags;
+		bool basic;
+	} flags[] = {
+		{ "", 0 , true},
+		{ "-noreloc", NORELOC, true },
+		{ "-active", ACTIVE, true },
+		{ "-hang", ACTIVE | HANG },
+		{ },
+	}, *f;
 	uint64_t size;
 	int fd = -1;
 
-	igt_fixture
+	igt_fixture {
 		fd = drm_open_driver_master(DRIVER_INTEL);
+		igt_require_gem(fd);
+	}
 
-	igt_subtest("basic-cpu")
-		basic_cpu(fd);
+	for (f = flags; f->name; f++) {
+		igt_hang_t hang;
 
-	igt_subtest("basic-gtt")
-		basic_gtt(fd);
+		igt_subtest_group {
+			igt_fixture {
+				if (f->flags & HANG)
+					hang = igt_allow_hang(fd, 0, 0);
+			}
 
-	igt_subtest("basic-noreloc")
-		basic_noreloc(fd);
+			for (m = modes; m->name; m++) {
+				igt_subtest_f("%s%s%s",
+					      f->basic ? "basic-" : "",
+					      m->name,
+					      f->name) {
+					if ((m->before | m->after) & I915_GEM_DOMAIN_WC)
+						igt_require(gem_mmap__has_wc(fd));
+					basic_reloc(fd, m->before, m->after, f->flags);
+				}
+			}
+
+			if (!(f->flags & NORELOC)) {
+				igt_subtest_f("%srange%s",
+					      f->basic ? "basic-" : "", f->name)
+					basic_range(fd, f->flags);
+			}
+
+			igt_fixture {
+				if (f->flags & HANG)
+					igt_disallow_hang(fd, hang);
+			}
+		}
+	}
 
 	igt_subtest("basic-softpin")
 		basic_softpin(fd);
@@ -589,8 +734,10 @@ igt_main
 			from_mmap(fd, size, MEM | RO);
 		igt_subtest_f("cpu-%u", find_last_set(size) - 1)
 			from_mmap(fd, size, CPU);
-		igt_subtest_f("wc-%u", find_last_set(size) - 1)
+		igt_subtest_f("wc-%u", find_last_set(size) - 1) {
+			igt_require(gem_mmap__has_wc(fd));
 			from_mmap(fd, size, WC);
+		}
 		igt_subtest_f("gtt-%u", find_last_set(size) - 1)
 			from_mmap(fd, size, GTT);
 	}
