@@ -60,17 +60,12 @@ static void check_bo(int fd, uint32_t handle)
 	munmap(map, 4096);
 }
 
-static bool can_mi_store_dword(int gen, unsigned engine)
-{
-	return gen > 2 && !(gen == 6 && (engine & ~(3<<13)) == I915_EXEC_BSD);
-}
-
-static bool ignore_engine(int gen, unsigned engine)
+static bool ignore_engine(int fd, unsigned engine)
 {
 	if (engine == 0)
 		return true;
 
-	if (!can_mi_store_dword(gen, engine))
+	if (!gem_can_store_dword(fd, engine))
 		return true;
 
 	return false;
@@ -78,11 +73,10 @@ static bool ignore_engine(int gen, unsigned engine)
 
 static void test_all(int fd, unsigned flags)
 {
-	const int gen = intel_gen(intel_get_drm_devid(fd));
 	unsigned engine;
 
 	for_each_engine(fd, engine) {
-		if (!ignore_engine(gen, engine))
+		if (!ignore_engine(fd, engine))
 			run_test(fd, engine, flags & ~0xff);
 	}
 }
@@ -122,17 +116,17 @@ static void run_test(int fd, unsigned engine, unsigned flags)
 		 */
 		if (has_semaphores(fd)) {
 			for_each_engine(fd, engine) {
-				if (!ignore_engine(gen, engine))
+				if (!ignore_engine(fd, engine))
 					engines[nengine++] = engine;
 			}
 		} else {
 			igt_require(gem_has_ring(fd, 0));
-			igt_require(can_mi_store_dword(gen, 0));
+			igt_require(gem_can_store_dword(fd, 0));
 			engines[nengine++] = 0;
 		}
 	} else {
 		igt_require(gem_has_ring(fd, engine));
-		igt_require(can_mi_store_dword(gen, engine));
+		igt_require(gem_can_store_dword(fd, engine));
 		engines[nengine++] = engine;
 	}
 	igt_require(nengine);
@@ -238,11 +232,6 @@ static void run_test(int fd, unsigned engine, unsigned flags)
 		test_all(fd, flags);
 }
 
-static bool can_store_dword_imm(int fd)
-{
-	return intel_gen(intel_get_drm_devid(fd)) > 2;
-}
-
 igt_main
 {
 	const struct {
@@ -260,7 +249,7 @@ igt_main
 	igt_fixture {
 		fd = drm_open_driver_master(DRIVER_INTEL);
 		igt_require_gem(fd);
-		igt_require(can_store_dword_imm(fd));
+		igt_require(gem_can_store_dword(fd, 0));
 
 		igt_fork_hang_detector(fd);
 	}
