@@ -33,6 +33,10 @@
 
 #include "igt.h"
 #include "igt_gt.h"
+#include "igt_vgem.h"
+
+#include <signal.h>
+#include <sys/ioctl.h>
 
 #define INTERRUPTIBLE 0x1
 #define HANG 0x2
@@ -41,6 +45,9 @@
 #define BOMB 0x10
 #define SUSPEND 0x20
 #define HIBERNATE 0x40
+#define NEWFD 0x80
+
+static unsigned int ring_size;
 
 static void check_bo(int fd, uint32_t handle)
 {
@@ -57,7 +64,7 @@ static void check_bo(int fd, uint32_t handle)
 
 static void fill_ring(int fd,
 		      struct drm_i915_gem_execbuffer2 *execbuf,
-		      unsigned flags)
+		      unsigned flags, unsigned timeout)
 {
 	/* The ring we've been using is 128k, and each rendering op
 	 * will use at least 8 dwords:
@@ -75,48 +82,47 @@ static void fill_ring(int fd,
 	 * doing this, we aren't likely to with this test.
 	 */
 	igt_debug("Executing execbuf %d times\n", 128*1024/(8*4));
-	igt_while_interruptible(flags & INTERRUPTIBLE) {
-		for (int i = 0; i < 128*1024 / (8 * 4); i++)
-			gem_execbuf(fd, execbuf);
+	igt_until_timeout(timeout) {
+		igt_while_interruptible(flags & INTERRUPTIBLE) {
+			for (typeof(ring_size) i = 0; i < ring_size; i++)
+				gem_execbuf(fd, execbuf);
+		}
 	}
 }
 
-static void run_test(int fd, unsigned ring, unsigned flags)
+static int setup_execbuf(int fd,
+			 struct drm_i915_gem_execbuffer2 *execbuf,
+			 struct drm_i915_gem_exec_object2 *obj,
+			 struct drm_i915_gem_relocation_entry *reloc,
+			 unsigned int ring)
 {
 	const int gen = intel_gen(intel_get_drm_devid(fd));
 	const uint32_t bbe = MI_BATCH_BUFFER_END;
-	struct drm_i915_gem_exec_object2 obj[2];
-	struct drm_i915_gem_relocation_entry reloc[1024];
-	struct drm_i915_gem_execbuffer2 execbuf;
-	igt_hang_t hang;
 	uint32_t *batch, *b;
+	int ret;
 	int i;
 
-	gem_require_ring(fd, ring);
-	igt_skip_on_f(gen == 6 && (ring & ~(3<<13)) == I915_EXEC_BSD,
-		      "MI_STORE_DATA broken on gen6 bsd\n");
+	memset(execbuf, 0, sizeof(*execbuf));
+	memset(obj, 0, 2*sizeof(*obj));
+	memset(reloc, 0, 1024*sizeof(*reloc));
 
-	if (flags & (SUSPEND | HIBERNATE))
-		run_test(fd, ring, 0);
+	execbuf->buffers_ptr = to_user_pointer(obj);
+	execbuf->flags = ring | (1 << 11) | (1 << 12);
 
-	gem_quiescent_gpu(fd);
+	if (gen > 3 && gen < 6)
+		execbuf->flags |= I915_EXEC_SECURE;
 
-	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)obj;
-	execbuf.flags = ring | (1 << 11) | (1 << 12);
-	if (gen < 6)
-		execbuf.flags |= I915_EXEC_SECURE;
-
-	memset(obj, 0, sizeof(obj));
 	obj[0].handle = gem_create(fd, 4096);
 	gem_write(fd, obj[0].handle, 0, &bbe, sizeof(bbe));
-	execbuf.buffer_count = 1;
-	igt_require(__gem_execbuf(fd, &execbuf) == 0);
+	execbuf->buffer_count = 1;
+	ret = __gem_execbuf(fd, execbuf);
+	if (ret)
+		return ret;
 
 	obj[0].flags |= EXEC_OBJECT_WRITE;
 	obj[1].handle = gem_create(fd, 1024*16 + 4096);
 
-	obj[1].relocs_ptr = (uintptr_t)reloc;
+	obj[1].relocs_ptr = to_user_pointer(reloc);
 	obj[1].relocation_count = 1024;
 
 	batch = gem_mmap__cpu(fd, obj[1].handle, 0, 16*1024 + 4096,
@@ -124,7 +130,6 @@ static void run_test(int fd, unsigned ring, unsigned flags)
 	gem_set_domain(fd, obj[1].handle,
 		       I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_CPU);
 
-	memset(reloc, 0, sizeof(reloc));
 	b = batch;
 	for (i = 0; i < 1024; i++) {
 		uint64_t offset;
@@ -136,15 +141,18 @@ static void run_test(int fd, unsigned ring, unsigned flags)
 		reloc[i].write_domain = I915_GEM_DOMAIN_INSTRUCTION;
 
 		offset = obj[0].offset + reloc[i].delta;
-		*b++ = MI_STORE_DWORD_IMM | (gen < 6 ? 1 << 22 : 0);
+		*b++ = MI_STORE_DWORD_IMM;
 		if (gen >= 8) {
 			*b++ = offset;
 			*b++ = offset >> 32;
 		} else if (gen >= 4) {
+			if (gen < 6)
+				b[-1] |= 1 << 22;
 			*b++ = 0;
 			*b++ = offset;
 			reloc[i].offset += sizeof(*batch);
 		} else {
+			b[-1] |= 1 << 22;
 			b[-1] -= 1;
 			*b++ = offset;
 		}
@@ -152,9 +160,29 @@ static void run_test(int fd, unsigned ring, unsigned flags)
 	}
 	*b++ = MI_BATCH_BUFFER_END;
 	munmap(batch, 16*1024+4096);
-	execbuf.buffer_count = 2;
-	gem_execbuf(fd, &execbuf);
+
+	execbuf->buffer_count = 2;
+	gem_execbuf(fd, execbuf);
+
 	check_bo(fd, obj[0].handle);
+	return 0;
+}
+
+static void run_test(int fd, unsigned ring, unsigned flags, unsigned timeout)
+{
+	struct drm_i915_gem_exec_object2 obj[2];
+	struct drm_i915_gem_relocation_entry reloc[1024];
+	struct drm_i915_gem_execbuffer2 execbuf;
+	igt_hang_t hang;
+
+	gem_require_ring(fd, ring);
+	igt_require(gem_can_store_dword(fd, ring));
+
+	if (flags & (SUSPEND | HIBERNATE))
+		run_test(fd, ring, 0, 0);
+
+	gem_quiescent_gpu(fd);
+	igt_require(setup_execbuf(fd, &execbuf, obj, reloc, ring) == 0);
 
 	memset(&hang, 0, sizeof(hang));
 	if (flags & HANG)
@@ -171,8 +199,13 @@ static void run_test(int fd, unsigned ring, unsigned flags)
 			nchild = 1;
 
 		igt_debug("Forking %d children\n", nchild);
-		igt_fork(child, nchild)
-			fill_ring(fd, &execbuf, flags);
+		igt_fork(child, nchild) {
+			if (flags & NEWFD) {
+				fd = drm_open_driver(DRIVER_INTEL);
+				setup_execbuf(fd, &execbuf, obj, reloc, ring);
+			}
+			fill_ring(fd, &execbuf, flags, timeout);
+		}
 
 		if (flags & SUSPEND)
 			igt_system_suspend_autoresume(SUSPEND_STATE_MEM,
@@ -182,9 +215,12 @@ static void run_test(int fd, unsigned ring, unsigned flags)
 			igt_system_suspend_autoresume(SUSPEND_STATE_DISK,
 						      SUSPEND_TEST_NONE);
 
+		if (flags & NEWFD)
+			fill_ring(fd, &execbuf, flags, timeout);
+
 		igt_waitchildren();
 	} else
-		fill_ring(fd, &execbuf, flags);
+		fill_ring(fd, &execbuf, flags, timeout);
 
 	if (flags & HANG)
 		igt_post_hang_ring(fd, hang);
@@ -197,7 +233,95 @@ static void run_test(int fd, unsigned ring, unsigned flags)
 	gem_quiescent_gpu(fd);
 
 	if (flags & (SUSPEND | HIBERNATE))
-		run_test(fd, ring, 0);
+		run_test(fd, ring, 0, 0);
+}
+
+struct cork {
+	int device;
+	uint32_t handle;
+	uint32_t fence;
+};
+
+static void plug(int fd, struct cork *c)
+{
+	struct vgem_bo bo;
+	int dmabuf;
+
+	c->device = drm_open_driver(DRIVER_VGEM);
+
+	bo.width = bo.height = 1;
+	bo.bpp = 4;
+	vgem_create(c->device, &bo);
+	c->fence = vgem_fence_attach(c->device, &bo, VGEM_FENCE_WRITE);
+
+	dmabuf = prime_handle_to_fd(c->device, bo.handle);
+	c->handle = prime_fd_to_handle(fd, dmabuf);
+	close(dmabuf);
+}
+
+static void unplug(struct cork *c)
+{
+	vgem_fence_signal(c->device, c->fence);
+	close(c->device);
+}
+
+static void alarm_handler(int sig)
+{
+}
+
+static int __execbuf(int fd, struct drm_i915_gem_execbuffer2 *execbuf)
+{
+	return ioctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, execbuf);
+}
+
+static unsigned int measure_ring_size(int fd)
+{
+	struct sigaction sa = { .sa_handler = alarm_handler };
+	struct drm_i915_gem_exec_object2 obj[2];
+	struct drm_i915_gem_execbuffer2 execbuf;
+	const uint32_t bbe = MI_BATCH_BUFFER_END;
+	unsigned int count, last;
+	struct itimerval itv;
+	struct cork c;
+
+	memset(obj, 0, sizeof(obj));
+	obj[1].handle = gem_create(fd, 4096);
+	gem_write(fd, obj[1].handle, 0, &bbe, sizeof(bbe));
+
+	plug(fd, &c);
+	obj[0].handle = c.handle;
+
+	memset(&execbuf, 0, sizeof(execbuf));
+	execbuf.buffers_ptr = to_user_pointer(obj);
+	execbuf.buffer_count = 2;
+
+	sigaction(SIGALRM, &sa, NULL);
+	itv.it_interval.tv_sec = 0;
+	itv.it_interval.tv_usec = 100;
+	itv.it_value.tv_sec = 0;
+	itv.it_value.tv_usec = 1000;
+	setitimer(ITIMER_REAL, &itv, NULL);
+
+	last = count = 0;
+	do {
+		if (__execbuf(fd, &execbuf) == 0) {
+			count++;
+			continue;
+		}
+
+		if (last == count)
+			break;
+
+		last = count;
+	} while (1);
+
+	memset(&itv, 0, sizeof(itv));
+	setitimer(ITIMER_REAL, &itv, NULL);
+
+	unplug(&c);
+	gem_close(fd, obj[1].handle);
+
+	return count;
 }
 
 igt_main
@@ -205,31 +329,52 @@ igt_main
 	const struct {
 		const char *suffix;
 		unsigned flags;
+		unsigned timeout;
 		bool basic;
 	} modes[] = {
-		{ "", 0, true},
-		{ "-interruptible", INTERRUPTIBLE, true },
-		{ "-hang", HANG, true },
-		{ "-child", CHILD },
-		{ "-forked", FORKED, true },
-		{ "-bomb", BOMB | INTERRUPTIBLE },
-		{ "-S3", BOMB | SUSPEND },
-		{ "-S4", BOMB | HIBERNATE },
+		{ "", 0, 0, true},
+		{ "-interruptible", INTERRUPTIBLE, 1, true },
+		{ "-hang", HANG, 10, true },
+		{ "-child", CHILD, 0 },
+		{ "-forked", FORKED, 0, true },
+		{ "-fd", FORKED | NEWFD, 0, true },
+		{ "-bomb", BOMB | NEWFD | INTERRUPTIBLE, 150 },
+		{ "-S3", BOMB | SUSPEND, 30 },
+		{ "-S4", BOMB | HIBERNATE, 30 },
 		{ NULL }
 	}, *m;
-	const struct intel_execution_engine *e;
-	int fd;
+	bool master = false;
+	int fd = -1;
 
-	igt_fixture
-		fd = drm_open_driver_master(DRIVER_INTEL);
+	igt_fixture {
+		int gen;
+
+		fd = drm_open_driver(DRIVER_INTEL);
+		igt_require_gem(fd);
+		igt_require(gem_can_store_dword(fd, 0));
+		gen = intel_gen(intel_get_drm_devid(fd));
+		if (gen > 3 && gen < 6) { /* ctg and ilk need secure batches */
+			igt_require(drmSetMaster(fd) == 0);
+			master = true;
+		}
+
+		ring_size = measure_ring_size(fd);
+		igt_info("Ring size: %d batches\n", ring_size);
+		igt_require(ring_size);
+	}
 
 	for (m = modes; m->suffix; m++) {
+		const struct intel_execution_engine *e;
+
 		for (e = intel_execution_engines; e->name; e++) {
 			igt_subtest_f("%s%s%s",
 				      m->basic && !e->exec_id ? "basic-" : "",
 				      e->name,
-				      m->suffix)
-				run_test(fd, e->exec_id | e->flags, m->flags);
+				      m->suffix) {
+				igt_skip_on(m->flags & NEWFD && master);
+				run_test(fd, e->exec_id | e->flags,
+					 m->flags, m->timeout);
+			}
 		}
 	}
 
