@@ -29,6 +29,8 @@
 
 #include "igt.h"
 #include "igt_gt.h"
+#include "igt_debugfs.h"
+#include "igt_sysfs.h"
 
 #ifndef MADV_FREE
 #define MADV_FREE 8
@@ -87,7 +89,7 @@ static void execbuf1(int fd, uint64_t alloc)
 
 	memset(&obj, 0, sizeof(obj));
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)&obj;
+	execbuf.buffers_ptr = to_user_pointer(&obj);
 	execbuf.buffer_count = 1;
 
 	obj.handle = gem_create(fd, alloc);
@@ -138,7 +140,7 @@ static void execbufN(int fd, uint64_t alloc)
 		int j = count - i;
 
 		obj[j].handle = gem_create(fd, 1 << 20);
-		execbuf.buffers_ptr = (uintptr_t)&obj[j];
+		execbuf.buffers_ptr = to_user_pointer(&obj[j]);
 		execbuf.buffer_count = i + 1;
 		gem_execbuf(fd, &execbuf);
 	}
@@ -166,7 +168,7 @@ static void hang(int fd, uint64_t alloc)
 		int j = count - i;
 
 		obj[j].handle = gem_create(fd, 1 << 20);
-		execbuf.buffers_ptr = (uintptr_t)&obj[j];
+		execbuf.buffers_ptr = to_user_pointer(&obj[j]);
 		execbuf.buffer_count = i + 1;
 		gem_execbuf(fd, &execbuf);
 	}
@@ -191,7 +193,7 @@ static void userptr(int fd, uint64_t alloc)
 
 	memset(&userptr, 0, sizeof(userptr));
 	userptr.user_size = alloc;
-	userptr.user_ptr = (uintptr_t)ptr;
+	userptr.user_ptr = to_user_pointer(ptr);
 	do_ioctl(fd, LOCAL_IOCTL_I915_GEM_USERPTR, &userptr);
 
 	gem_set_domain(fd, userptr.handle, I915_GEM_DOMAIN_GTT, 0);
@@ -291,6 +293,44 @@ static void run_test(int nchildren, uint64_t alloc,
 	igt_waitchildren();
 }
 
+static void reclaim(unsigned engine, int timeout)
+{
+	const uint64_t timeout_100ms = 100000000LL;
+	int fd = drm_open_driver(DRIVER_INTEL);
+	int debugfs = igt_debugfs_dir(fd);
+	igt_spin_t *spin;
+	volatile uint32_t *shared;
+
+	shared = mmap(0, 4096, PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
+	igt_assert(shared != MAP_FAILED);
+
+	igt_fork(child, sysconf(_SC_NPROCESSORS_ONLN)) {
+		do {
+			igt_sysfs_printf(debugfs, "i915_drop_caches",
+					"%d", DROP_BOUND | DROP_UNBOUND);
+		} while (!*shared);
+	}
+
+	spin = igt_spin_batch_new(fd, engine, 0);
+	igt_until_timeout(timeout) {
+		igt_spin_t *next = __igt_spin_batch_new(fd, engine, 0);
+
+		igt_spin_batch_set_timeout(spin, timeout_100ms);
+		gem_sync(fd, spin->handle);
+
+		igt_spin_batch_free(fd, spin);
+		spin = next;
+	}
+	igt_spin_batch_free(fd, spin);
+
+	*shared = 1;
+	igt_waitchildren();
+
+	munmap((void *)shared, 4096);
+	close(debugfs);
+	close(fd);
+}
+
 igt_main
 {
 	const struct test {
@@ -341,6 +381,9 @@ igt_main
 				     CHECK_SWAP | CHECK_RAM);
 	}
 
+	igt_subtest("reclaim")
+		reclaim(I915_EXEC_DEFAULT, 2);
+
 	for(const struct test *t = tests; t->name; t++) {
 		for(const struct mode *m = modes; m->suffix; m++) {
 			igt_subtest_f("%s%s", t->name, m->suffix)
