@@ -31,6 +31,7 @@ typedef struct {
 	int gfx_fd;
 	igt_display_t display;
 	struct igt_fb fb;
+	struct igt_fb fb_reference;
 	struct igt_fb fb_modeset;
 	struct igt_fb fb_flip;
 	igt_crc_t ref_crc;
@@ -38,19 +39,18 @@ typedef struct {
 	igt_rotation_t rotation;
 	int pos_x;
 	int pos_y;
-	unsigned int w, h;
 	uint32_t override_fmt;
 	uint64_t override_tiling;
 	unsigned int flip_stress;
 } data_t;
 
 static void
-paint_squares(data_t *data, drmModeModeInfo *mode, igt_rotation_t rotation,
+paint_squares(data_t *data, igt_rotation_t rotation,
 	      struct igt_fb *fb, float o)
 {
 	cairo_t *cr;
-	unsigned int w = data->w;
-	unsigned int h = data->h;
+	unsigned int w = fb->width;
+	unsigned int h = fb->height;
 
 	cr = igt_get_cairo_ctx(data->gfx_fd, fb);
 
@@ -84,30 +84,43 @@ paint_squares(data_t *data, drmModeModeInfo *mode, igt_rotation_t rotation,
 	cairo_destroy(cr);
 }
 
-static void commit_crtc(data_t *data, igt_output_t *output, igt_plane_t *plane)
+static void prepare_crtc(data_t *data, igt_output_t *output, enum pipe pipe,
+			 igt_plane_t *plane, enum igt_commit_style commit)
 {
+	drmModeModeInfo *mode;
+	unsigned int w, h;
+	uint64_t tiling = data->override_tiling ?: LOCAL_DRM_FORMAT_MOD_NONE;
+	uint32_t pixel_format = data->override_fmt ?: DRM_FORMAT_XRGB8888;
 	igt_display_t *display = &data->display;
-	enum igt_commit_style commit = COMMIT_LEGACY;
-	igt_plane_t *primary;
+	igt_plane_t *primary = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
+
+	igt_output_set_pipe(output, pipe);
+	igt_plane_set_rotation(plane, IGT_ROTATION_0);
+
+	/* create the pipe_crc object for this pipe */
+	igt_pipe_crc_free(data->pipe_crc);
+	data->pipe_crc = igt_pipe_crc_new(data->gfx_fd, pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
+
+	mode = igt_output_get_mode(output);
+
+	w = mode->hdisplay;
+	h = mode->vdisplay;
+
+	igt_create_fb(data->gfx_fd, w, h, pixel_format, tiling, &data->fb_modeset);
 
 	/*
 	 * With igt_display_commit2 and COMMIT_UNIVERSAL, we call just the
 	 * setplane without a modeset. So, to be able to call
 	 * igt_display_commit and ultimately setcrtc to do the first modeset,
 	 * we create an fb covering the crtc and call commit
+	 *
+	 * It's also a good idea to set a primary fb on the primary plane
+	 * regardless, to force a underrun when watermarks are allocated
+	 * incorrectly for other planes.
 	 */
-
-	if (plane->type == DRM_PLANE_TYPE_PRIMARY ||
-	    plane->type == DRM_PLANE_TYPE_CURSOR)
-		commit = COMMIT_UNIVERSAL;
-
-	if (data->display.is_atomic)
-		commit = COMMIT_ATOMIC;
-
-	primary = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
+	igt_plane_set_fb(primary, &data->fb_modeset);
 
 	if (commit < COMMIT_ATOMIC) {
-		igt_plane_set_fb(primary, &data->fb_modeset);
 		primary->rotation_changed = false;
 		igt_display_commit(display);
 
@@ -115,43 +128,80 @@ static void commit_crtc(data_t *data, igt_output_t *output, igt_plane_t *plane)
 			primary->rotation_changed = true;
 	}
 
-	igt_plane_set_fb(plane, &data->fb);
-
-	if (plane->type != DRM_PLANE_TYPE_CURSOR)
-		igt_plane_set_position(plane, data->pos_x, data->pos_y);
+	igt_plane_set_fb(plane, NULL);
 
 	igt_display_commit2(display, commit);
 }
 
-static void prepare_crtc(data_t *data, igt_output_t *output, enum pipe pipe,
-			 igt_plane_t *plane)
+static void remove_fbs(data_t *data)
+{
+	if (!data->fb.fb_id)
+		return;
+
+	igt_remove_fb(data->gfx_fd, &data->fb);
+	igt_remove_fb(data->gfx_fd, &data->fb_reference);
+
+	if (data->fb_flip.fb_id)
+		igt_remove_fb(data->gfx_fd, &data->fb_flip);
+
+	data->fb_flip.fb_id = data->fb.fb_id = 0;
+}
+
+enum rectangle_type {
+	square,
+	portrait,
+	landscape,
+	num_rectangle_types /* must be last */
+};
+
+static void prepare_fbs(data_t *data, igt_output_t *output,
+			igt_plane_t *plane, enum rectangle_type rect)
 {
 	drmModeModeInfo *mode;
-	int fb_id, fb_modeset_id;
-	unsigned int w, h;
-	uint64_t tiling = data->override_tiling ?
-			  data->override_tiling : LOCAL_DRM_FORMAT_MOD_NONE;
-	uint32_t pixel_format = data->override_fmt ?
-				data->override_fmt : DRM_FORMAT_XRGB8888;
+	igt_display_t *display = &data->display;
+	unsigned int w, h, ref_w, ref_h, min_w, min_h;
+	uint64_t tiling = data->override_tiling ?: LOCAL_DRM_FORMAT_MOD_NONE;
+	uint32_t pixel_format = data->override_fmt ?: DRM_FORMAT_XRGB8888;
+
+	if (data->fb.fb_id) {
+		igt_plane_set_fb(plane, NULL);
+		igt_display_commit2(display, display->is_atomic ? COMMIT_ATOMIC : COMMIT_UNIVERSAL);
+
+		remove_fbs(data);
+	}
 
-	igt_output_set_pipe(output, pipe);
 	igt_plane_set_rotation(plane, IGT_ROTATION_0);
 
-	/* create the pipe_crc object for this pipe */
-	igt_pipe_crc_free(data->pipe_crc);
-	data->pipe_crc = igt_pipe_crc_new(pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
-
 	mode = igt_output_get_mode(output);
+	if (plane->type != DRM_PLANE_TYPE_CURSOR) {
+		w = mode->hdisplay;
+		h = mode->vdisplay;
 
-	w = mode->hdisplay;
-	h = mode->vdisplay;
+		min_w = 256;
+		min_h = 256;
+	} else {
+		pixel_format = data->override_fmt ?: DRM_FORMAT_ARGB8888;
 
-	fb_modeset_id = igt_create_fb(data->gfx_fd,
-				      w, h,
-				      pixel_format,
-				      tiling,
-				      &data->fb_modeset);
-	igt_assert(fb_modeset_id);
+		w = h = 256;
+		min_w = min_h = 64;
+	}
+
+	switch (rect) {
+	case square:
+		w = h = min(h, w);
+		break;
+	case portrait:
+		w = min_w;
+		break;
+	case landscape:
+		h = min_h;
+		break;
+	case num_rectangle_types:
+		igt_assert(0);
+	}
+
+	ref_w = w;
+	ref_h = h;
 
 	/*
 	 * For 90/270, we will use create smaller fb so that the rotated
@@ -159,46 +209,40 @@ static void prepare_crtc(data_t *data, igt_output_t *output, enum pipe pipe,
 	 */
 	if (data->rotation == IGT_ROTATION_90 ||
 	    data->rotation == IGT_ROTATION_270) {
-		tiling = data->override_tiling ?
-			 data->override_tiling : LOCAL_I915_FORMAT_MOD_Y_TILED;
-		w = h = min(mode->hdisplay, mode->vdisplay);
-	} else if (plane->type == DRM_PLANE_TYPE_CURSOR) {
-		pixel_format = data->override_fmt ?
-			       data->override_fmt : DRM_FORMAT_ARGB8888;
-		w = h = 128;
+		tiling = data->override_tiling ?: LOCAL_I915_FORMAT_MOD_Y_TILED;
+
+		igt_swap(w, h);
 	}
 
-	data->w = w;
-	data->h = h;
-
-	fb_id = igt_create_fb(data->gfx_fd,
-			      w, h,
-			      pixel_format,
-			      tiling,
-			      &data->fb);
-	igt_assert(fb_id);
+	igt_create_fb(data->gfx_fd, w, h, pixel_format, tiling, &data->fb);
 
 	if (data->flip_stress) {
-		fb_id = igt_create_fb(data->gfx_fd,
-				      w, h,
-				      pixel_format,
-				      tiling,
-				      &data->fb_flip);
-		igt_assert(fb_id);
-		paint_squares(data, mode, IGT_ROTATION_0, &data->fb_flip, 0.92);
+		igt_create_fb(data->gfx_fd, w, h, pixel_format, tiling, &data->fb_flip);
+		paint_squares(data, IGT_ROTATION_0, &data->fb_flip, 0.92);
 	}
 
 	/* Step 1: create a reference CRC for a software-rotated fb */
-	paint_squares(data, mode, data->rotation, &data->fb, 1.0);
-	commit_crtc(data, output, plane);
+	igt_create_fb(data->gfx_fd, ref_w, ref_h, pixel_format,
+		      data->override_tiling ?: LOCAL_DRM_FORMAT_MOD_NONE, &data->fb_reference);
+	paint_squares(data, data->rotation, &data->fb_reference, 1.0);
+
+	igt_plane_set_fb(plane, &data->fb_reference);
+	if (plane->type != DRM_PLANE_TYPE_CURSOR)
+		igt_plane_set_position(plane, data->pos_x, data->pos_y);
+	igt_plane_set_rotation(plane, IGT_ROTATION_0);
+	igt_display_commit2(display, display->is_atomic ? COMMIT_ATOMIC : COMMIT_UNIVERSAL);
+
 	igt_pipe_crc_collect_crc(data->pipe_crc, &data->ref_crc);
 
 	/*
 	 * Step 2: prepare the plane with an non-rotated fb let the hw
 	 * rotate it.
 	 */
-	paint_squares(data, mode, IGT_ROTATION_0, &data->fb, 1.0);
+	paint_squares(data, IGT_ROTATION_0, &data->fb, 1.0);
 	igt_plane_set_fb(plane, &data->fb);
+
+	if (plane->type != DRM_PLANE_TYPE_CURSOR)
+		igt_plane_set_position(plane, data->pos_x, data->pos_y);
 }
 
 static void cleanup_crtc(data_t *data, igt_output_t *output, igt_plane_t *plane)
@@ -208,10 +252,9 @@ static void cleanup_crtc(data_t *data, igt_output_t *output, igt_plane_t *plane)
 	igt_pipe_crc_free(data->pipe_crc);
 	data->pipe_crc = NULL;
 
-	igt_remove_fb(data->gfx_fd, &data->fb);
+	remove_fbs(data);
+
 	igt_remove_fb(data->gfx_fd, &data->fb_modeset);
-	if (data->fb_flip.fb_id)
-		igt_remove_fb(data->gfx_fd, &data->fb_flip);
 
 	/* XXX: see the note in prepare_crtc() */
 	if (plane->type != DRM_PLANE_TYPE_PRIMARY) {
@@ -233,7 +276,7 @@ static void cleanup_crtc(data_t *data, igt_output_t *output, igt_plane_t *plane)
 
 static void wait_for_pageflip(int fd)
 {
-	drmEventContext evctx = { .version = DRM_EVENT_CONTEXT_VERSION };
+	drmEventContext evctx = { .version = 2 };
 	struct timeval timeout = { .tv_sec = 0, .tv_usec = 50000 };
 	fd_set fds;
 	int ret;
@@ -270,47 +313,61 @@ static void test_plane_rotation(data_t *data, int plane_type)
 
 	for_each_pipe_with_valid_output(display, pipe, output) {
 		igt_plane_t *plane;
+		int i;
 
 		igt_output_set_pipe(output, pipe);
 
 		plane = igt_output_get_plane_type(output, plane_type);
 		igt_require(igt_plane_supports_rotation(plane));
 
-		prepare_crtc(data, output, pipe, plane);
+		prepare_crtc(data, output, pipe, plane, commit);
 
-		igt_display_commit2(display, commit);
+		for (i = 0; i < num_rectangle_types; i++) {
+			/* Unsupported on i915 */
+			if (plane_type == DRM_PLANE_TYPE_CURSOR &&
+			    i != square)
+				continue;
 
-		/* collect unrotated CRC */
-		igt_pipe_crc_collect_crc(data->pipe_crc, &crc_unrotated);
+			igt_debug("Testing case %i on pipe %s\n", i, kmstest_pipe_name(pipe));
+			prepare_fbs(data, output, plane, i);
 
-		igt_plane_set_rotation(plane, data->rotation);
-		ret = igt_display_try_commit2(display, commit);
-		if (data->override_fmt || data->override_tiling) {
-			igt_assert_eq(ret, -EINVAL);
-		} else {
-			igt_assert_eq(ret, 0);
-			igt_pipe_crc_collect_crc(data->pipe_crc,
-						  &crc_output);
-			igt_assert_crc_equal(&data->ref_crc,
-					      &crc_output);
-		}
+			igt_display_commit2(display, commit);
 
-		flip_count = data->flip_stress;
-		while (flip_count--) {
-			ret = drmModePageFlip(data->gfx_fd,
-					      output->config.crtc->crtc_id,
-					      data->fb_flip.fb_id,
-					      DRM_MODE_PAGE_FLIP_EVENT,
-					      NULL);
-			igt_assert_eq(ret, 0);
-			wait_for_pageflip(data->gfx_fd);
-			ret = drmModePageFlip(data->gfx_fd,
-					      output->config.crtc->crtc_id,
-					      data->fb.fb_id,
-					      DRM_MODE_PAGE_FLIP_EVENT,
-					      NULL);
-			igt_assert_eq(ret, 0);
-			wait_for_pageflip(data->gfx_fd);
+			/* collect unrotated CRC */
+			igt_pipe_crc_collect_crc(data->pipe_crc, &crc_unrotated);
+
+			igt_plane_set_rotation(plane, data->rotation);
+			if (data->rotation == IGT_ROTATION_90 || data->rotation == IGT_ROTATION_270)
+				igt_plane_set_size(plane, data->fb.height, data->fb.width);
+
+			ret = igt_display_try_commit2(display, commit);
+			if (data->override_fmt || data->override_tiling) {
+				igt_assert_eq(ret, -EINVAL);
+			} else {
+				igt_assert_eq(ret, 0);
+				igt_pipe_crc_collect_crc(data->pipe_crc,
+							  &crc_output);
+				igt_assert_crc_equal(&data->ref_crc,
+						      &crc_output);
+			}
+
+			flip_count = data->flip_stress;
+			while (flip_count--) {
+				ret = drmModePageFlip(data->gfx_fd,
+						      output->config.crtc->crtc_id,
+						      data->fb_flip.fb_id,
+						      DRM_MODE_PAGE_FLIP_EVENT,
+						      NULL);
+				igt_assert_eq(ret, 0);
+				wait_for_pageflip(data->gfx_fd);
+				ret = drmModePageFlip(data->gfx_fd,
+						      output->config.crtc->crtc_id,
+						      data->fb.fb_id,
+						      DRM_MODE_PAGE_FLIP_EVENT,
+						      NULL);
+				igt_assert_eq(ret, 0);
+				wait_for_pageflip(data->gfx_fd);
+			}
 		}
 
 		valid_tests++;
@@ -517,7 +574,7 @@ igt_main
 
 		kmstest_set_vt_graphics_mode();
 
-		igt_require_pipe_crc();
+		igt_require_pipe_crc(data.gfx_fd);
 
 		igt_display_init(&data.display, data.gfx_fd);
 	}
