@@ -31,15 +31,13 @@
 #include <unistd.h>
 #include <string.h>
 #include <sys/time.h>
-
+#include <math.h>
 #include "intel_bufmgr.h"
 
 #define MAX_CONNECTORS  10
-#define MAX_CRTCS       3
+#define MAX_CRTCS       6
 
 /* max combinations with repetitions */
-#define MAX_COMBINATION_COUNT   \
-	(MAX_CONNECTORS * MAX_CONNECTORS * MAX_CONNECTORS)
 #define MAX_COMBINATION_ELEMS   MAX_CRTCS
 
 static int drm_fd;
@@ -71,6 +69,7 @@ enum test_flags {
 	TEST_SINGLE_CRTC_CLONE		= 0x04,
 	TEST_EXCLUSIVE_CRTC_CLONE	= 0x08,
 	TEST_STEALING			= 0x10,
+	TEST_TIMINGS			= 0x20,
 };
 
 struct test_config {
@@ -82,7 +81,6 @@ struct test_config {
 struct connector_config {
 	drmModeConnector *connector;
 	int crtc_idx;
-	bool connected;
 	drmModeModeInfo default_mode;
 };
 
@@ -249,10 +247,9 @@ static void get_crtc_config_str(struct crtc_config *crtc, char *buf,
 		drmModeConnector *connector = crtc->cconfs[i].connector;
 
 		pos += snprintf(&buf[pos], buf_size - pos,
-			"%s%s-%d[%d]%s", i ? ", " : "",
+			"%s%s-%d[%d]", i ? ", " : "",
 			kmstest_connector_type_str(connector->connector_type),
-			connector->connector_type_id, connector->connector_id,
-			crtc->cconfs[i].connected ? "" : " (NC)");
+			connector->connector_type_id, connector->connector_id);
 		if (pos > buf_size)
 			return;
 	}
@@ -268,6 +265,8 @@ static void setup_crtcs(drmModeRes *resources, struct connector_config *cconf,
 	int i;
 	int encoder_usage_count[resources->count_encoders];
 
+	kmstest_unset_all_crtcs(drm_fd, resources);
+
 	i = 0;
 	crtc_count = 0;
 	crtc = crtcs;
@@ -409,6 +408,117 @@ static int test_stealing(int fd, struct crtc_config *crtc, uint32_t *ids)
 	return ret;
 }
 
+static double frame_time(const drmModeModeInfo *kmode)
+{
+	return 1000.0 * kmode->htotal * kmode->vtotal / kmode->clock;
+}
+
+static double line_time(const drmModeModeInfo *kmode)
+{
+	return 1000.0 * kmode->htotal / kmode->clock;
+}
+
+static void check_timings(int crtc_idx, const drmModeModeInfo *kmode)
+{
+#define CALIBRATE_TS_STEPS 120 /* ~2s has to be less than 128! */
+	drmVBlank wait;
+	igt_stats_t stats;
+	uint32_t last_seq;
+	uint64_t last_timestamp;
+	double expected;
+	double accuracy;
+	double mean;
+	double stddev;
+	int n;
+
+	memset(&wait, 0, sizeof(wait));
+	wait.request.type = kmstest_get_vbl_flag(crtc_idx);
+	wait.request.type |= DRM_VBLANK_ABSOLUTE | DRM_VBLANK_NEXTONMISS;
+	do_or_die(drmWaitVBlank(drm_fd, &wait));
+
+	last_seq = wait.reply.sequence;
+	last_timestamp = wait.reply.tval_sec;
+	last_timestamp *= 1000000;
+	last_timestamp += wait.reply.tval_usec;
+
+	memset(&wait, 0, sizeof(wait));
+	wait.request.type = kmstest_get_vbl_flag(crtc_idx);
+	wait.request.type |= DRM_VBLANK_ABSOLUTE | DRM_VBLANK_EVENT;
+	wait.request.sequence = last_seq;
+	for (n = 0; n < CALIBRATE_TS_STEPS; n++) {
+		++wait.request.sequence;
+		do_or_die(drmWaitVBlank(drm_fd, &wait));
+	}
+
+	igt_stats_init_with_size(&stats, CALIBRATE_TS_STEPS);
+	for (n = 0; n < CALIBRATE_TS_STEPS; n++) {
+		struct drm_event_vblank ev;
+		uint64_t now;
+
+		igt_assert(read(drm_fd, &ev, sizeof(ev)) == sizeof(ev));
+		igt_assert_eq(ev.sequence, last_seq + 1);
+
+		now = ev.tv_sec;
+		now *= 1000000;
+		now += ev.tv_usec;
+
+		igt_stats_push(&stats, now - last_timestamp);
+
+		last_timestamp = now;
+		last_seq = ev.sequence;
+	}
+
+	expected = frame_time(kmode);
+
+	mean = igt_stats_get_mean(&stats);
+	stddev = igt_stats_get_std_deviation(&stats);
+
+	/* 99.7% samples fall within `accuracy` on both sides of mean in normal
+	 * distribution if `accuracy = 3 * sigma`.
+	 * https://en.wikipedia.org/wiki/68%E2%80%9395%E2%80%9399.7_rule
+	 *
+	 * The value of 99.7% was chosen to suit requirements of test cases
+	 * which depend on timing, giving the lowest acceptable MTBF of 5.6s
+	 * for 60Hz sampling rate.
+	 */
+	accuracy = 3. * stddev;
+
+	igt_info("Expected frametime: %.0fus; measured %.1fus +- %.3fus accuracy %.2f%% [%.2f scanlines]\n",
+		 expected, mean, stddev,
+		 100 * accuracy / mean, accuracy / line_time(kmode));
+
+	/* 99.7% samples within one scanline on each side of mean */
+	igt_assert_f(accuracy < line_time(kmode),
+		     "vblank accuracy (%.3fus, %.1f%%) worse than a scanline (%.3fus)\n",
+		     accuracy, 100 * accuracy / mean, line_time(kmode));
+
+	/* At least 90% of frame times fall within the one scanline on each
+	 * side of expected mean.
+	 *
+	 * Expected scanline duration:
+	 * 	(expected - accuracy, expected + accuracy).
+	 * Assuming maximum difference allowed:
+	 * 	expected = mean + n * sigma
+	 * the scanline duration becomes:
+	 * 	(mean - accuracy + n * sigma, mean + accuracy + n * sigma)
+	 * The expected scanline captures the following number of samples
+	 * from each side of expected:
+	 * 	(erf(abs(-(accuracy/sigma) + n) / sqrt(2))
+	 * 	+ erf((accuracy/sigma) + n) / sqrt(2))) / 2
+	 * 	= samples
+	 *
+	 * Solving for samples = 0.9:
+	 * 	n = 1.718
+	 *
+	 * See:
+	 * https://en.wikipedia.org/wiki/Standard_deviation#Rules_for_normally_distributed_data
+	 */
+	igt_assert_f(fabs(mean - expected) < 1.718 * stddev,
+		     "vblank interval differs from modeline! expected %.1fus, measured %1.fus +- %.3fus, difference %.1fus (%.1f sigma)\n",
+		     expected, mean, stddev,
+		     fabs(mean - expected), fabs(mean - expected) / stddev);
+}
+
 static void test_crtc_config(const struct test_config *tconf,
 			     struct crtc_config *crtcs, int crtc_count)
 {
@@ -417,7 +527,6 @@ static void test_crtc_config(const struct test_config *tconf,
 	struct crtc_config *crtc;
 	static int test_id;
 	bool config_failed = false;
-	bool connector_connected = false;
 	int ret = 0;
 	int i;
 
@@ -441,7 +550,6 @@ static void test_crtc_config(const struct test_config *tconf,
 
 	for (i = 0; i < crtc_count; i++) {
 		uint32_t *ids;
-		int j;
 
 		crtc = &crtcs[i];
 
@@ -464,15 +572,12 @@ static void test_crtc_config(const struct test_config *tconf,
 			igt_assert_eq(errno, EINVAL);
 			config_failed = true;
 		}
-
-		for (j = 0; j < crtc->connector_count; j++)
-			connector_connected |= crtc->cconfs[j].connected;
 	}
 
 	igt_assert(config_failed == !!(tconf->flags & TEST_INVALID));
 
-	if (ret == 0 && connector_connected && !(tconf->flags & TEST_INVALID))
-		sleep(5);
+	if (ret == 0 && tconf->flags & TEST_TIMINGS)
+		check_timings(crtcs[0].crtc_idx, &crtcs[0].mode);
 
 	for (i = 0; i < crtc_count; i++) {
 		if (crtcs[i].fb_info.fb_id) {
@@ -534,37 +639,22 @@ static int get_one_connector(drmModeRes *resources, int connector_id,
 			     struct connector_config *cconf)
 {
 	drmModeConnector *connector;
-	drmModeModeInfo mode;
 
 	connector = drmModeGetConnectorCurrent(drm_fd, connector_id);
 	igt_assert(connector);
 	cconf->connector = connector;
 
-	cconf->connected = connector->connection == DRM_MODE_CONNECTED;
-
-	/*
-	 * For DP/eDP we need a connected sink, since mode setting depends
-	 * on successful link training and retrieved DPCD parameters.
-	 */
-	switch (connector->connector_type) {
-	case DRM_MODE_CONNECTOR_DisplayPort:
-	case DRM_MODE_CONNECTOR_eDP:
-		if (!cconf->connected) {
-			drmModeFreeConnector(connector);
-			return -1;
-		}
+	if (connector->connection != DRM_MODE_CONNECTED) {
+		drmModeFreeConnector(connector);
+		return -1;
 	}
 
-	if (cconf->connected) {
-		if (!kmstest_get_connector_default_mode(drm_fd, connector,
-							&mode))
-			mode = mode_640_480;
-	} else {
-		mode = mode_640_480;
+	if (!kmstest_get_connector_default_mode(drm_fd, connector,
+						&cconf->default_mode)) {
+		drmModeFreeConnector(connector);
+		return -1;
 	}
 
-	cconf->default_mode = mode;
-
 	return 0;
 }
 
@@ -610,7 +700,8 @@ struct combination {
 
 struct combination_set {
 	int count;
-	struct combination items[MAX_COMBINATION_COUNT];
+	int capacity;
+	struct combination *items;
 };
 
 /*
@@ -624,7 +715,7 @@ static void iterate_combinations(int n, int k, bool allow_repetitions,
 	int v;
 
 	if (!k) {
-		igt_assert(set->count < ARRAY_SIZE(set->items));
+		igt_assert(set->count < set->capacity);
 		set->items[set->count++] = *comb;
 		return;
 	}
@@ -659,6 +750,16 @@ static void test_combinations(const struct test_config *tconf,
 	if (connector_count > 2 && (tconf->flags & TEST_STEALING))
 		return;
 
+	igt_assert(tconf->resources);
+
+	connector_combs.capacity = pow(tconf->resources->count_connectors,
+				       tconf->resources->count_crtcs + 1);
+	crtc_combs.capacity = pow(tconf->resources->count_crtcs,
+				  tconf->resources->count_crtcs + 1);
+
+	connector_combs.items = malloc(connector_combs.capacity * sizeof(struct combination));
+	crtc_combs.items = malloc(crtc_combs.capacity * sizeof(struct combination));
+
 	get_combinations(tconf->resources->count_connectors, connector_count,
 			 false, &connector_combs);
 	get_combinations(tconf->resources->count_crtcs, connector_count,
@@ -695,6 +796,9 @@ static void test_combinations(const struct test_config *tconf,
 free_cconfs:
 		free(cconfs);
 	}
+
+	free(connector_combs.items);
+	free(crtc_combs.items);
 }
 
 static void run_test(const struct test_config *tconf)
@@ -728,6 +832,7 @@ int main(int argc, char **argv)
 		enum test_flags flags;
 		const char *name;
 	} tests[] = {
+		{ TEST_TIMINGS, "basic" },
 		{ TEST_CLONE | TEST_SINGLE_CRTC_CLONE,
 					"basic-clone-single-crtc" },
 		{ TEST_INVALID | TEST_CLONE | TEST_SINGLE_CRTC_CLONE,
