@@ -32,6 +32,7 @@
 #include <sys/types.h>
 #include <unistd.h>
 
+#include "igt.h"
 #include "intel_io.h"
 #include "intel_chipset.h"
 
@@ -57,6 +58,7 @@ struct config {
 	struct pci_device *pci_dev;
 	char *mmiofile;
 	uint32_t devid;
+	int drm_fd;
 
 	/* read: number of registers to read */
 	uint32_t count;
@@ -409,7 +411,7 @@ static int intel_reg_read(struct config *config, int argc, char *argv[])
 	if (config->mmiofile)
 		intel_mmio_use_dump_file(config->mmiofile);
 	else
-		intel_register_access_init(config->pci_dev, 0);
+		intel_register_access_init(config->pci_dev, 0, config->drm_fd);
 
 	for (i = 1; i < argc; i++) {
 		struct reg reg;
@@ -439,7 +441,7 @@ static int intel_reg_write(struct config *config, int argc, char *argv[])
 		return EXIT_FAILURE;
 	}
 
-	intel_register_access_init(config->pci_dev, 0);
+	intel_register_access_init(config->pci_dev, 0, config->drm_fd);
 
 	for (i = 1; i < argc; i += 2) {
 		struct reg reg;
@@ -477,7 +479,7 @@ static int intel_reg_dump(struct config *config, int argc, char *argv[])
 	if (config->mmiofile)
 		intel_mmio_use_dump_file(config->mmiofile);
 	else
-		intel_register_access_init(config->pci_dev, 0);
+		intel_register_access_init(config->pci_dev, 0, config->drm_fd);
 
 	for (i = 0; i < config->regcount; i++) {
 		reg = &config->regs[i];
@@ -491,7 +493,7 @@ static int intel_reg_dump(struct config *config, int argc, char *argv[])
 
 	intel_register_access_fini();
 
-	return EXIT_FAILURE;
+	return EXIT_SUCCESS;
 }
 
 static int intel_reg_snapshot(struct config *config, int argc, char *argv[])
@@ -657,24 +659,7 @@ static int intel_reg_help(struct config *config, int argc, char *argv[])
  */
 static const char *get_codename(uint32_t devid)
 {
-	if (IS_GEN5(devid))
-		return "ironlake";
-	else if (IS_GEN6(devid))
-		return "sandybridge";
-	else if (IS_IVYBRIDGE(devid))
-		return "ivybridge";
-	else if (IS_HASWELL(devid))
-		return "haswell";
-	else if (IS_BROADWELL(devid))
-		return "broadwell";
-	else if (IS_SKYLAKE(devid))
-		return "skylake";
-	else if (IS_CHERRYVIEW(devid))
-		return "cherryview";
-	else if (IS_VALLEYVIEW(devid))
-		return "valleyview";
-
-	return NULL;
+	return intel_get_device_info(devid)->codename;
 }
 
 /*
@@ -894,6 +879,9 @@ int main(int argc, char *argv[])
 		config.devid = config.pci_dev->device_id;
 	}
 
+	/* Just to make sure we open the right debugfs files */
+	config.drm_fd = __drm_open_driver(DRIVER_INTEL);
+
 	if (read_reg_spec(&config) < 0) {
 		return EXIT_FAILURE;
 	}
